@@ -49,7 +49,6 @@ Torus Knot Software Ltd.
 #include "OgreEdgeListBuilder.h"
 #include "OgreStringConverter.h"
 #include "OgreAnimation.h"
-#include "OgreAlignedAllocator.h"
 #include "OgreOptimisedUtil.h"
 #include "OgreSceneNode.h"
 
@@ -83,12 +82,12 @@ namespace Ogre {
           mSkeletonInstance(0),
 		  mInitialised(false),
 		  mLastParentXform(Matrix4::ZERO),
-          mFullBoundingBox(),
-		  mNormaliseNormals(false)
+		  mMeshStateCount(0),
+          mFullBoundingBox()
     {
     }
     //-----------------------------------------------------------------------
-    Entity::Entity( const String& name, MeshPtr& mesh) :
+    Entity::Entity( const String& name, const MeshPtr& mesh) :
 		MovableObject(name),
         mMesh(mesh),
         mAnimationState(NULL),
@@ -117,8 +116,8 @@ namespace Ogre {
 		mSkeletonInstance(0),
 		mInitialised(false),
 		mLastParentXform(Matrix4::ZERO),
-        mFullBoundingBox(),
-		mNormaliseNormals(false)
+		mMeshStateCount(0),
+        mFullBoundingBox()
 	{
 		_initialise();
     }
@@ -158,7 +157,7 @@ namespace Ogre {
 		// Is mesh skeletally animated?
 		if (mMesh->hasSkeleton() && !mMesh->getSkeleton().isNull())
 		{
-			mSkeletonInstance = new SkeletonInstance(mMesh->getSkeleton());
+			mSkeletonInstance = OGRE_NEW SkeletonInstance(mMesh->getSkeleton());
 			mSkeletonInstance->load();
 		}
 
@@ -175,7 +174,7 @@ namespace Ogre {
 			{
 				const MeshLodUsage& usage = mMesh->getLodLevel(i);
 				// Manually create entity
-				Entity* lodEnt = new Entity(mName + "Lod" + StringConverter::toString(i),
+				Entity* lodEnt = OGRE_NEW Entity(mName + "Lod" + StringConverter::toString(i),
 					usage.manualMesh);
 				mLodEntityList.push_back(lodEnt);
 			}
@@ -185,13 +184,13 @@ namespace Ogre {
 		// Initialise the AnimationState, if Mesh has animation
 		if (hasSkeleton())
 		{
-			mFrameBonesLastUpdated = new unsigned long(std::numeric_limits<unsigned long>::max());
+			mFrameBonesLastUpdated = OGRE_NEW_T(unsigned long, MEMCATEGORY_ANIMATION)(std::numeric_limits<unsigned long>::max());
 			mNumBoneMatrices = mSkeletonInstance->getNumBones();
-			mBoneMatrices = static_cast<Matrix4*>(AlignedMemory::allocate(sizeof(Matrix4) * mNumBoneMatrices));
+			mBoneMatrices = static_cast<Matrix4*>(OGRE_MALLOC_SIMD(sizeof(Matrix4) * mNumBoneMatrices, MEMCATEGORY_ANIMATION));
 		}
 		if (hasSkeleton() || hasVertexAnimation())
 		{
-			mAnimationState = new AnimationStateSet();
+			mAnimationState = OGRE_NEW AnimationStateSet();
 			mMesh->_initAnimationState(mAnimationState);
 			prepareTempBlendBuffers();
 		}
@@ -206,6 +205,7 @@ namespace Ogre {
 		}
 
 		mInitialised = true;
+		mMeshStateCount = mMesh->getStateCount();
 
 	}
 	//-----------------------------------------------------------------------
@@ -220,7 +220,7 @@ namespace Ogre {
 		for (i = mSubEntityList.begin(); i != iend; ++i)
 		{
 			// Delete SubEntity
-			delete *i;
+			OGRE_DELETE *i;
 		}
 		mSubEntityList.clear();
 		
@@ -230,7 +230,7 @@ namespace Ogre {
 		for (li = mLodEntityList.begin(); li != liend; ++li)
 		{
 			// Delete
-			delete (*li);
+			OGRE_DELETE (*li);
 		}
         mLodEntityList.clear();
         
@@ -239,7 +239,7 @@ namespace Ogre {
 		siend = mShadowRenderables.end();
 		for (si = mShadowRenderables.begin(); si != siend; ++si)
 		{
-			delete *si;
+			OGRE_DELETE *si;
 		}
         mShadowRenderables.clear();
         
@@ -248,7 +248,7 @@ namespace Ogre {
 		detachAllObjectsImpl();
 
 		if (mSkeletonInstance) {
-			AlignedMemory::deallocate(mBoneWorldMatrices);
+			OGRE_FREE_SIMD(mBoneWorldMatrices, MEMCATEGORY_ANIMATION);
 
             if (mSharedSkeletonEntities) {
                 mSharedSkeletonEntities->erase(this);
@@ -259,27 +259,29 @@ namespace Ogre {
                 // Should never occuring, just in case
                 else if (mSharedSkeletonEntities->empty())
                 {
-                    delete mSharedSkeletonEntities;
-                    delete mFrameBonesLastUpdated;
-                    delete mSkeletonInstance;
-                    AlignedMemory::deallocate(mBoneMatrices);
-                    delete mAnimationState;
+                    OGRE_DELETE_T(mSharedSkeletonEntities, EntitySet, MEMCATEGORY_ANIMATION);
+					// using OGRE_FREE since unsigned long is not a destructor
+                    OGRE_FREE(mFrameBonesLastUpdated, MEMCATEGORY_ANIMATION);
+                    OGRE_DELETE mSkeletonInstance;
+                    OGRE_FREE_SIMD(mBoneMatrices, MEMCATEGORY_ANIMATION);
+                    OGRE_DELETE mAnimationState;
                 }
             } else {
-                delete mFrameBonesLastUpdated;
-                delete mSkeletonInstance;
-                AlignedMemory::deallocate(mBoneMatrices);
-                delete mAnimationState;
+				// using OGRE_FREE since unsigned long is not a destructor
+				OGRE_FREE(mFrameBonesLastUpdated, MEMCATEGORY_ANIMATION);
+                OGRE_DELETE mSkeletonInstance;
+                OGRE_FREE_SIMD(mBoneMatrices, MEMCATEGORY_ANIMATION);
+                OGRE_DELETE mAnimationState;
             }
         }
 		else if (hasVertexAnimation())
 		{
-			delete mAnimationState;
+			OGRE_DELETE mAnimationState;
 		}
 
-		delete mSkelAnimVertexData;
-		delete mSoftwareVertexAnimVertexData;
-		delete mHardwareVertexAnimVertexData;
+		OGRE_DELETE mSkelAnimVertexData;
+		OGRE_DELETE mSoftwareVertexAnimVertexData;
+		OGRE_DELETE mHardwareVertexAnimVertexData;
 
 		mInitialised = false;
 	}
@@ -342,8 +344,8 @@ namespace Ogre {
 			}
 			if (mAnimationState)
 			{
-				delete newEnt->mAnimationState;
-				newEnt->mAnimationState = new AnimationStateSet(*mAnimationState);
+				OGRE_DELETE newEnt->mAnimationState;
+				newEnt->mAnimationState = OGRE_NEW AnimationStateSet(*mAnimationState);
 			}
 		}
 
@@ -360,6 +362,18 @@ namespace Ogre {
         }
 
     }
+
+
+	void Entity::setMaterial( const MaterialPtr& material )
+	{
+		// Set for all subentities
+		SubEntityList::iterator i;
+		for (i = mSubEntityList.begin(); i != mSubEntityList.end(); ++i)
+		{
+			(*i)->setMaterial(material);
+		}
+	}
+
     //-----------------------------------------------------------------------
     void Entity::_notifyCurrentCamera(Camera* cam)
     {
@@ -368,13 +382,14 @@ namespace Ogre {
         // Calculate the LOD
         if (mParentNode)
         {
-            Real squaredDepth = mParentNode->getSquaredViewDepth(cam);
+			const Camera* lodCamera = cam->getLodCamera();
+            Real squaredDepth = mParentNode->getSquaredViewDepth(lodCamera);
 
             // Do Mesh LOD
             // Adjust this depth by the entity bias factor
             Real tmp = squaredDepth * mMeshLodFactorInv;
             // Now adjust it by the camera bias
-            tmp = tmp * cam->_getLodBiasInverse();
+            tmp = tmp * lodCamera->_getLodBiasInverse();
             // Get the index at this biased depth
             mMeshLodIndex = mMesh->getLodIndexSquaredDepth(tmp);
             // Apply maximum detail restriction (remember lower = higher detail)
@@ -386,7 +401,7 @@ namespace Ogre {
             // Adjust this depth by the entity bias factor
             tmp = squaredDepth * mMaterialLodFactorInv;
             // Now adjust it by the camera bias
-            tmp = tmp * cam->_getLodBiasInverse();
+            tmp = tmp * lodCamera->_getLodBiasInverse();
             SubEntityList::iterator i, iend;
             iend = mSubEntityList.end();
             for (i = mSubEntityList.begin(); i != iend; ++i)
@@ -482,7 +497,15 @@ namespace Ogre {
 		if (!mInitialised)
 			return;
 
-        // Check we're not using a manual LOD
+		// Check mesh state count, will be incremented if reloaded
+		if (mMesh->getStateCount() != mMeshStateCount)
+		{
+			// force reinitialise
+			_initialise(true);
+		}
+
+        Entity* displayEntity = this;
+		// Check we're not using a manual LOD
         if (mMeshLodIndex > 0 && mMesh->isLodManual())
         {
             // Use alternate entity
@@ -496,14 +519,13 @@ namespace Ogre {
                 mAnimationState->copyMatchingState(
 					mLodEntityList[mMeshLodIndex - 1]->mAnimationState);
             }
-            mLodEntityList[mMeshLodIndex - 1]->_updateRenderQueue(queue);
-            return;
+            displayEntity = mLodEntityList[mMeshLodIndex - 1];
         }
 
         // Add each visible SubEntity to the queue
         SubEntityList::iterator i, iend;
-        iend = mSubEntityList.end();
-        for (i = mSubEntityList.begin(); i != iend; ++i)
+        iend = displayEntity->mSubEntityList.end();
+        for (i = displayEntity->mSubEntityList.begin(); i != iend; ++i)
         {
             if((*i)->isVisible())
             {
@@ -520,17 +542,34 @@ namespace Ogre {
 
         // Since we know we're going to be rendered, take this opportunity to
         // update the animation
-        if (hasSkeleton() || hasVertexAnimation())
+        if (displayEntity->hasSkeleton() || displayEntity->hasVertexAnimation())
         {
-            updateAnimation();
+            displayEntity->updateAnimation();
 
             //--- pass this point,  we are sure that the transformation matrix of each bone and tagPoint have been updated
             ChildObjectList::iterator child_itr = mChildObjectList.begin();
             ChildObjectList::iterator child_itr_end = mChildObjectList.end();
             for( ; child_itr != child_itr_end; child_itr++)
             {
-                if ((*child_itr).second->isVisible())
-                    (*child_itr).second->_updateRenderQueue(queue);
+                MovableObject* child = child_itr->second;
+                bool isVisible = child->isVisible();
+                if (isVisible && (displayEntity != this))
+                {
+                    //Check if the bone exists in the current LOD
+
+                    //The child is connected to a tagpoint which is connected to a bone
+                    Bone* bone = static_cast<Bone*>(child->getParentNode()->getParent());
+                    if (!displayEntity->getSkeleton()->hasBone(bone->getName()))
+                    {
+                        //Current LOD entity does not have the bone that the
+                        //child is connected to. Do not display.
+                        isVisible = false;
+                    }
+                }
+                if (isVisible)
+                {
+                    child->_updateRenderQueue(queue);
+                }   
             }
         }
 
@@ -774,7 +813,7 @@ namespace Ogre {
                 if (!mBoneWorldMatrices)
                 {
                     mBoneWorldMatrices =
-                        static_cast<Matrix4*>(AlignedMemory::allocate(sizeof(Matrix4) * mNumBoneMatrices));
+                        static_cast<Matrix4*>(OGRE_MALLOC_SIMD(sizeof(Matrix4) * mNumBoneMatrices, MEMCATEGORY_ANIMATION));
                 }
 
                 OptimisedUtil::getImplementation()->concatenateAffineMatrices(
@@ -1052,7 +1091,7 @@ namespace Ogre {
     void Entity::cacheBoneMatrices(void)
     {
         Root& root = Root::getSingleton();
-        unsigned long currentFrameNumber = root.getCurrentFrameNumber();
+        unsigned long currentFrameNumber = root.getNextFrameNumber();
         if (*mFrameBonesLastUpdated  != currentFrameNumber) {
 
             mSkeletonInstance->setAnimationState(*mAnimationState);
@@ -1112,7 +1151,7 @@ namespace Ogre {
         for (i = 0; i < numSubMeshes; ++i)
         {
             subMesh = mesh->getSubMesh(i);
-            subEnt = new SubEntity(this, subMesh);
+            subEnt = OGRE_NEW SubEntity(this, subMesh);
             if (subMesh->isMatInitialised())
                 subEnt->setMaterialName(subMesh->getMaterialName());
             sublist->push_back(subEnt);
@@ -1270,17 +1309,17 @@ namespace Ogre {
     {
         if (mSkelAnimVertexData)
         {
-            delete mSkelAnimVertexData;
+            OGRE_DELETE mSkelAnimVertexData;
             mSkelAnimVertexData = 0;
         }
 		if (mSoftwareVertexAnimVertexData)
 		{
-			delete mSoftwareVertexAnimVertexData;
+			OGRE_DELETE mSoftwareVertexAnimVertexData;
 			mSoftwareVertexAnimVertexData = 0;
 		}
 		if (mHardwareVertexAnimVertexData)
 		{
-			delete mHardwareVertexAnimVertexData;
+			OGRE_DELETE mHardwareVertexAnimVertexData;
 			mHardwareVertexAnimVertexData = 0;
 		}
 
@@ -1396,18 +1435,18 @@ namespace Ogre {
             const MaterialPtr& m = sub->getMaterial();
             // Make sure it's loaded
             m->load();
-            Technique* t = m->getBestTechnique();
-            if (!t)
-            {
-                // No supported techniques
-                continue;
-            }
+            Technique* t = m->getBestTechnique(0, sub);
+			if (!t)
+			{
+				// No supported techniques
+				continue;
+			}
+			if (t->getNumPasses() == 0)
+			{
+				// No passes, invalid
+				continue;
+			}
             Pass* p = t->getPass(0);
-            if (!p)
-            {
-                // No passes, invalid
-                continue;
-            }
             if (p->hasVertexProgram())
             {
                 // If one material uses a vertex program, set this flag
@@ -1593,7 +1632,7 @@ namespace Ogre {
                 // for extruding the shadow volume) since otherwise we can
                 // get depth-fighting on the light cap
 
-                *si = new EntityShadowRenderable(this, indexBuffer, pVertData,
+                *si = OGRE_NEW EntityShadowRenderable(this, indexBuffer, pVertData,
                     mVertexProgramInUse || !extrude, subent);
             }
             else
@@ -1749,13 +1788,13 @@ namespace Ogre {
         mCurrentVertexData = vertexData;
 
         // Initialise render op
-        mRenderOp.indexData = new IndexData();
+        mRenderOp.indexData = OGRE_NEW IndexData();
         mRenderOp.indexData->indexBuffer = *indexBuffer;
         mRenderOp.indexData->indexStart = 0;
         // index start and count are sorted out later
 
         // Create vertex data which just references position component (and 2 component)
-        mRenderOp.vertexData = new VertexData();
+        mRenderOp.vertexData = OGRE_NEW VertexData();
         // Map in position data
         mRenderOp.vertexData->vertexDeclaration->addElement(0,0,VET_FLOAT3, VES_POSITION);
         mOriginalPosBufferBinding =
@@ -1786,7 +1825,7 @@ namespace Ogre {
             if (createSeparateLightCap)
             {
                 // Create child light cap
-                mLightCap = new EntityShadowRenderable(parent,
+                mLightCap = OGRE_NEW EntityShadowRenderable(parent,
                     indexBuffer, vertexData, false, subent, true);
             }
         }
@@ -1795,8 +1834,8 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Entity::EntityShadowRenderable::~EntityShadowRenderable()
     {
-        delete mRenderOp.indexData;
-        delete mRenderOp.vertexData;
+        OGRE_DELETE mRenderOp.indexData;
+        OGRE_DELETE mRenderOp.vertexData;
     }
     //-----------------------------------------------------------------------
     void Entity::EntityShadowRenderable::getWorldTransforms(Matrix4* xform) const
@@ -1804,16 +1843,6 @@ namespace Ogre {
         *xform = mParent->_getParentNodeFullTransform();
     }
     //-----------------------------------------------------------------------
-    const Quaternion& Entity::EntityShadowRenderable::getWorldOrientation(void) const
-    {
-        return mParent->getParentNode()->_getDerivedOrientation();
-    }
-    //-----------------------------------------------------------------------
-    const Vector3& Entity::EntityShadowRenderable::getWorldPosition(void) const
-    {
-        return mParent->getParentNode()->_getDerivedPosition();
-    }
-    //-----------------------------------------------------------------------
     void Entity::EntityShadowRenderable::rebindPositionBuffer(const VertexData* vertexData, bool force)
     {
         if (force || mCurrentVertexData != vertexData)
@@ -1886,10 +1915,11 @@ namespace Ogre {
         }
         else
         {
-            delete mSkeletonInstance;
-            AlignedMemory::deallocate(mBoneMatrices);
-            delete mAnimationState;
-            delete mFrameBonesLastUpdated;
+            OGRE_DELETE mSkeletonInstance;
+            OGRE_FREE_SIMD(mBoneMatrices, MEMCATEGORY_ANIMATION);
+            OGRE_DELETE mAnimationState;
+			// using OGRE_FREE since unsigned long is not a destructor
+			OGRE_FREE(mFrameBonesLastUpdated, MEMCATEGORY_ANIMATION);
             mSkeletonInstance = entity->mSkeletonInstance;
             mNumBoneMatrices = entity->mNumBoneMatrices;
             mBoneMatrices = entity->mBoneMatrices;
@@ -1897,7 +1927,7 @@ namespace Ogre {
             mFrameBonesLastUpdated = entity->mFrameBonesLastUpdated;
             if (entity->mSharedSkeletonEntities == NULL)
             {
-                entity->mSharedSkeletonEntities = new EntitySet();
+                entity->mSharedSkeletonEntities = OGRE_NEW_T(EntitySet, MEMCATEGORY_ANIMATION)();
                 entity->mSharedSkeletonEntities->insert(entity);
             }
             mSharedSkeletonEntities = entity->mSharedSkeletonEntities;
@@ -1917,18 +1947,18 @@ namespace Ogre {
         if (mSharedSkeletonEntities->size() == 1)
         {
             //just reset
-            delete mSharedSkeletonEntities;
+            OGRE_DELETE_T(mSharedSkeletonEntities, EntitySet, MEMCATEGORY_ANIMATION);
             mSharedSkeletonEntities = 0;
         }
         else
         {
-            mSkeletonInstance = new SkeletonInstance(mMesh->getSkeleton());
+            mSkeletonInstance = OGRE_NEW SkeletonInstance(mMesh->getSkeleton());
             mSkeletonInstance->load();
-            mAnimationState = new AnimationStateSet();
+            mAnimationState = OGRE_NEW AnimationStateSet();
             mMesh->_initAnimationState(mAnimationState);
-            mFrameBonesLastUpdated = new unsigned long(std::numeric_limits<unsigned long>::max());
+            mFrameBonesLastUpdated = OGRE_NEW_T(unsigned long, MEMCATEGORY_ANIMATION)(std::numeric_limits<unsigned long>::max());
             mNumBoneMatrices = mSkeletonInstance->getNumBones();
-            mBoneMatrices = static_cast<Matrix4*>(AlignedMemory::allocate(sizeof(Matrix4) * mNumBoneMatrices));
+            mBoneMatrices = static_cast<Matrix4*>(OGRE_MALLOC_SIMD(sizeof(Matrix4) * mNumBoneMatrices, MEMCATEGORY_ANIMATION));
 
             mSharedSkeletonEntities->erase(this);
             if (mSharedSkeletonEntities->size() == 1)
@@ -2007,6 +2037,29 @@ namespace Ogre {
 			return BIND_ORIGINAL;
 		}
 
+	}
+	//---------------------------------------------------------------------
+	void Entity::visitRenderables(Renderable::Visitor* visitor, 
+		bool debugRenderables)
+	{
+		// Visit each SubEntity
+		for (SubEntityList::iterator i = mSubEntityList.begin(); i != mSubEntityList.end(); ++i)
+		{
+			visitor->visit(*i, 0, false);
+		}
+		// if manual LOD is in use, visit those too
+		ushort lodi = 1;
+		for (LODEntityList::iterator e = mLodEntityList.begin(); 
+			e != mLodEntityList.end(); ++e, ++lodi)
+		{
+			
+			uint nsub = (*e)->getNumSubEntities();
+			for (uint s = 0; s < nsub; ++s)
+			{
+				visitor->visit((*e)->getSubEntity(s), lodi, false);
+			}
+		}
+
 	}
 	//-----------------------------------------------------------------------
 	//-----------------------------------------------------------------------
@@ -2042,13 +2095,13 @@ namespace Ogre {
 				"EntityFactory::createInstance");
 		}
 
-		return new Entity(name, pMesh);
+		return OGRE_NEW Entity(name, pMesh);
 
 	}
 	//-----------------------------------------------------------------------
 	void EntityFactory::destroyInstance( MovableObject* obj)
 	{
-		delete obj;
+		OGRE_DELETE obj;
 	}
 
 
