@@ -118,12 +118,35 @@ namespace Ogre {
 
 
     //-----------------------------------------------------------------------
-    void Material::loadImpl(void)
+    void Material::prepareImpl(void)
     {
 		// compile if required
         if (mCompilationRequired)
             compile();
 
+        // Load all supported techniques
+        Techniques::iterator i, iend;
+        iend = mSupportedTechniques.end();
+        for (i = mSupportedTechniques.begin(); i != iend; ++i)
+        {
+            (*i)->_prepare();
+        }
+    }
+    //-----------------------------------------------------------------------
+    void Material::unprepareImpl(void)
+    {
+        // Load all supported techniques
+        Techniques::iterator i, iend;
+        iend = mSupportedTechniques.end();
+        for (i = mSupportedTechniques.begin(); i != iend; ++i)
+        {
+            (*i)->_unprepare();
+        }
+    }
+    //-----------------------------------------------------------------------
+    void Material::loadImpl(void)
+    {
+
         // Load all supported techniques
         Techniques::iterator i, iend;
         iend = mSupportedTechniques.end();
@@ -224,7 +247,7 @@ namespace Ogre {
     //-----------------------------------------------------------------------
     Technique* Material::createTechnique(void)
     {
-        Technique *t = new Technique(this);
+        Technique *t = OGRE_NEW Technique(this);
         mTechniques.push_back(t);
         mCompilationRequired = true;
         return t;
@@ -306,7 +329,7 @@ namespace Ogre {
 		LodTechniques* lodtechs = 0;
 		if (i == mBestTechniquesBySchemeList.end())
 		{
-			lodtechs = new LodTechniques();
+			lodtechs = OGRE_NEW_T(LodTechniques, MEMCATEGORY_RESOURCE);
 			mBestTechniquesBySchemeList[schemeIndex] = lodtechs;
 		}
 		else
@@ -320,7 +343,7 @@ namespace Ogre {
 
 	}
 	//-----------------------------------------------------------------------------
-    Technique* Material::getBestTechnique(unsigned short lodIndex)
+    Technique* Material::getBestTechnique(unsigned short lodIndex, const Renderable* rend)
     {
         if (mSupportedTechniques.empty())
         {
@@ -329,13 +352,19 @@ namespace Ogre {
         else
         {
 			Technique* ret = 0;
+			MaterialManager& matMgr = MaterialManager::getSingleton();
 			// get scheme
 			BestTechniquesBySchemeList::iterator si = 
-				mBestTechniquesBySchemeList.find(
-				MaterialManager::getSingleton()._getActiveSchemeIndex());
+				mBestTechniquesBySchemeList.find(matMgr._getActiveSchemeIndex());
 			// scheme not found?
 			if (si == mBestTechniquesBySchemeList.end())
 			{
+				// listener specified alternative technique available?
+				ret = matMgr._arbitrateMissingTechniqueForActiveScheme(this, lodIndex, rend);
+				if (ret)
+					return ret;
+
+				// Nope, use default
 				// get the first item, will be 0 (the default) if default
 				// scheme techniques exist, otherwise the earliest defined
 				si = mBestTechniquesBySchemeList.begin();
@@ -380,7 +409,7 @@ namespace Ogre {
     {
         assert (index < mTechniques.size() && "Index out of bounds.");
         Techniques::iterator i = mTechniques.begin() + index;
-        delete(*i);
+        OGRE_DELETE(*i);
         mTechniques.erase(i);
         mSupportedTechniques.clear();
 		clearBestTechniqueList();
@@ -393,7 +422,7 @@ namespace Ogre {
         iend = mTechniques.end();
         for (i = mTechniques.begin(); i != iend; ++i)
         {
-            delete(*i);
+            OGRE_DELETE(*i);
         }
         mTechniques.clear();
         mSupportedTechniques.clear();
@@ -460,10 +489,9 @@ namespace Ogre {
         // Did we find any?
         if (mSupportedTechniques.empty())
         {
-			StringUtil::StrStreamType str;
-			str << "WARNING: material " << mName << " has no supportable "
-				"Techniques and will be blank. Explanation: " << std::endl << mUnsupportedReasons;
-            LogManager::getSingleton().logMessage(str.str());               
+			LogManager::getSingleton().stream()
+				<< "WARNING: material " << mName << " has no supportable "
+				<< "Techniques and will be blank. Explanation: \n" << mUnsupportedReasons;
         }
     }
 	//-----------------------------------------------------------------------
@@ -472,7 +500,7 @@ namespace Ogre {
 		for (BestTechniquesBySchemeList::iterator i = mBestTechniquesBySchemeList.begin();
 			i != mBestTechniquesBySchemeList.end(); ++i)
 		{
-			delete i->second;
+			OGRE_DELETE_T(i->second, LodTechniques, MEMCATEGORY_RESOURCE);
 		}
 		mBestTechniquesBySchemeList.clear();
 	}
@@ -690,6 +718,16 @@ namespace Ogre {
         }
     }
     // --------------------------------------------------------------------
+    void Material::setSeparateSceneBlending( const SceneBlendType sbt, const SceneBlendType sbta )
+    {
+        Techniques::iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            (*i)->setSeparateSceneBlending(sbt, sbta);
+        }
+    }
+    // --------------------------------------------------------------------
     void Material::setSceneBlending( const SceneBlendFactor sourceFactor, 
         const SceneBlendFactor destFactor)
     {
@@ -701,6 +739,16 @@ namespace Ogre {
         }
     }
     // --------------------------------------------------------------------
+    void Material::setSeparateSceneBlending( const SceneBlendFactor sourceFactor, const SceneBlendFactor destFactor, const SceneBlendFactor sourceFactorAlpha, const SceneBlendFactor destFactorAlpha)
+	{
+        Techniques::iterator i, iend;
+        iend = mTechniques.end();
+        for (i = mTechniques.begin(); i != iend; ++i)
+        {
+            (*i)->setSeparateSceneBlending(sourceFactor, destFactor, sourceFactorAlpha, destFactorAlpha);
+        }
+	}
+    // --------------------------------------------------------------------
     void Material::_notifyNeedsRecompile(void)
     {
         mCompilationRequired = true;
