@@ -49,6 +49,7 @@ namespace Ogre {
         , mParentNode(0)
         , mParentIsTagPoint(false)
         , mVisible(true)
+		, mDebugDisplay(false)
         , mUpperDistance(0)
         , mSquaredUpperDistance(0)
         , mBeyondFarDistance(false)
@@ -70,6 +71,7 @@ namespace Ogre {
         , mParentNode(0)
         , mParentIsTagPoint(false)
         , mVisible(true)
+		, mDebugDisplay(false)
         , mUpperDistance(0)
         , mSquaredUpperDistance(0)
         , mBeyondFarDistance(false)
@@ -156,6 +158,23 @@ namespace Ogre {
         return (mParentNode != 0);
 
     }
+	//---------------------------------------------------------------------
+	void MovableObject::detatchFromParent(void)
+	{
+		if (isAttached())
+		{
+			if (mParentIsTagPoint)
+			{
+				TagPoint* tp = static_cast<TagPoint*>(mParentNode);
+				tp->getParentEntity()->detachObjectFromBone(this);
+			}
+			else
+			{
+				SceneNode* sn = static_cast<SceneNode*>(mParentNode);
+				sn->detachObject(this);
+			}
+		}
+	}
     //-----------------------------------------------------------------------
 	bool MovableObject::isInScene(void) const
 	{
@@ -220,7 +239,7 @@ namespace Ogre {
 			if (cam->getUseRenderingDistance() && mUpperDistance > 0)
 			{
 				Real rad = getBoundingRadius();
-				Real squaredDepth = mParentNode->getSquaredViewDepth(cam);
+				Real squaredDepth = mParentNode->getSquaredViewDepth(cam->getLodCamera());
 				// Max distance to still render
 				Real maxDist = mUpperDistance + rad;
 				if (squaredDepth > Math::Sqr(maxDist))
