@@ -103,7 +103,8 @@ namespace Ogre {
                 "Unable to load world geometry. Invalid extension (must be .bsp).",
                 "BspSceneManager::setWorldGeometry");
 
-        strcpy(extension, filename.substr(pos + 1, filename.length() - pos).c_str());
+        strncpy(extension, filename.substr(pos + 1, filename.length() - pos).c_str(), 5);
+		extension[5] = 0;
 
         if (stricmp(extension, "bsp"))
             OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
@@ -131,7 +132,7 @@ namespace Ogre {
         // Init static render operation
         mRenderOp.vertexData = mLevel->mVertexData;
         // index data is per-frame
-        mRenderOp.indexData = new IndexData();
+        mRenderOp.indexData = OGRE_NEW IndexData();
         mRenderOp.indexData->indexStart = 0;
         mRenderOp.indexData->indexCount = 0;
         // Create enough index space to render whole level
@@ -173,7 +174,7 @@ namespace Ogre {
         // Init static render operation
         mRenderOp.vertexData = mLevel->mVertexData;
         // index data is per-frame
-        mRenderOp.indexData = new IndexData();
+        mRenderOp.indexData = OGRE_NEW IndexData();
         mRenderOp.indexData->indexStart = 0;
         mRenderOp.indexData->indexCount = 0;
         // Create enough index space to render whole level
@@ -203,6 +204,19 @@ namespace Ogre {
         // Movables are now added to the render queue in processVisibleLeaf
         walkTree(cam, &(findIt->second), onlyShadowCasters);
     }
+	//---------------------------------------------------------------------
+	bool BspSceneManager::fireRenderQueueEnded(uint8 id, const String& invocation)
+	{
+		bool repeat = SceneManager::fireRenderQueueEnded(id, invocation);
+		// Trigger level render just after skies
+		// can't trigger on mWorldGeometryRenderQueue because we're not queueing there
+		if (id == RENDER_QUEUE_SKIES_EARLY)
+		{
+			renderStaticGeometry();
+		}
+		return repeat;
+
+	}
     //-----------------------------------------------------------------------
     void BspSceneManager::renderStaticGeometry(void)
     {
@@ -217,7 +231,7 @@ namespace Ogre {
         // no world transform required
         mDestRenderSystem->_setWorldMatrix(Matrix4::IDENTITY);
         // Set view / proj
-        mDestRenderSystem->_setViewMatrix(mCameraInProgress->getViewMatrix(true));
+        mDestRenderSystem->_setViewMatrix(mCachedViewMatrix);
         mDestRenderSystem->_setProjectionMatrix(mCameraInProgress->getProjectionMatrixRS());
 
         // For each material in turn, cache rendering data & render
@@ -269,17 +283,6 @@ namespace Ogre {
         }
         */
     }
-    //-----------------------------------------------------------------------
-    void BspSceneManager::_renderVisibleObjects(void)
-    {
-        // Render static level geometry first
-        renderStaticGeometry();
-
-        // Call superclass to render the rest
-        SceneManager::_renderVisibleObjects();
-
-    }
-
     //-----------------------------------------------------------------------
     // REMOVE THIS CRAP
     //-----------------------------------------------------------------------
@@ -501,7 +504,7 @@ namespace Ogre {
     void BspSceneManager::freeMemory(void)
     {
         // no need to delete index buffer, will be handled by shared pointer
-        delete mRenderOp.indexData;
+        OGRE_DELETE mRenderOp.indexData;
 		mRenderOp.indexData = 0;
     }
     //-----------------------------------------------------------------------
@@ -606,18 +609,14 @@ namespace Ogre {
 
     }
     //-----------------------------------------------------------------------
-    SceneNode * BspSceneManager::createSceneNode( void )
+    SceneNode * BspSceneManager::createSceneNodeImpl( void )
     {
-        BspSceneNode * sn = new BspSceneNode( this );
-        mSceneNodes[ sn->getName() ] = sn;
-        return sn;
+        return OGRE_NEW BspSceneNode( this );
     }
     //-----------------------------------------------------------------------
-    SceneNode * BspSceneManager::createSceneNode( const String &name )
+    SceneNode * BspSceneManager::createSceneNodeImpl( const String &name )
     {
-        BspSceneNode * sn = new BspSceneNode( this, name );
-        mSceneNodes[ sn->getName() ] = sn;
-        return sn;
+        return OGRE_NEW BspSceneNode( this, name );
     }
     //-----------------------------------------------------------------------
     void BspSceneManager::_notifyObjectMoved(const MovableObject* mov, 
@@ -664,7 +663,7 @@ namespace Ogre {
     RaySceneQuery* BspSceneManager::
     createRayQuery(const Ray& ray, unsigned long mask)
     {
-        BspRaySceneQuery* q = new BspRaySceneQuery(this);
+        BspRaySceneQuery* q = OGRE_NEW BspRaySceneQuery(this);
         q->setRay(ray);
         q->setQueryMask(mask);
         return q;
@@ -673,7 +672,7 @@ namespace Ogre {
     IntersectionSceneQuery* BspSceneManager::
     createIntersectionQuery(unsigned long mask)
     {
-        BspIntersectionSceneQuery* q = new BspIntersectionSceneQuery(this);
+        BspIntersectionSceneQuery* q = OGRE_NEW BspIntersectionSceneQuery(this);
         q->setQueryMask(mask);
         return q;
     }
@@ -819,7 +818,7 @@ namespace Ogre {
         std::vector<WorldFragment*>::iterator i;
         for (i = mSingleIntersections.begin(); i != mSingleIntersections.end(); ++i)
         {
-            delete *i;
+            OGRE_FREE(*i, MEMCATEGORY_SCENE_CONTROL);
         }
         mSingleIntersections.clear();
     }
@@ -932,7 +931,7 @@ namespace Ogre {
                     {
                         // We're interested in a single intersection
                         // Have to create these 
-                        SceneQuery::WorldFragment* wf = new SceneQuery::WorldFragment();
+                        SceneQuery::WorldFragment* wf = OGRE_ALLOC_T(SceneQuery::WorldFragment, 1, MEMCATEGORY_SCENE_CONTROL);
                         wf->fragmentType = SceneQuery::WFT_SINGLE_INTERSECTION;
                         wf->singleIntersection = tracingRay.getPoint(result.second);
                         // save this so we can clean up later
@@ -975,12 +974,12 @@ namespace Ogre {
 	SceneManager* BspSceneManagerFactory::createInstance(
 		const String& instanceName)
 	{
-		return new BspSceneManager(instanceName);
+		return OGRE_NEW BspSceneManager(instanceName);
 	}
 	//-----------------------------------------------------------------------
 	void BspSceneManagerFactory::destroyInstance(SceneManager* instance)
 	{
-		delete instance;
+		OGRE_DELETE instance;
 	}
 
 }
