@@ -36,18 +36,17 @@ Torus Knot Software Ltd.
 #include "OgreImageCodec.h"
 #include "OgreStringConverter.h"
 
-#include "OgreNoMemoryMacros.h"
 #include <d3d9.h>
-#include "OgreMemoryMacros.h"
 #include "OgreRoot.h"
 #include "OgreWindowEventUtilities.h"
 
 namespace Ogre
 {
 	D3D9RenderWindow::D3D9RenderWindow(HINSTANCE instance, D3D9Driver *driver, LPDIRECT3DDEVICE9 deviceIfSwapChain)
-        : mInstance(instance)
-        , mDriver(driver)
-        , mpRenderSurface(0)
+		: mInstance(instance)
+		, mDriver(driver)
+		, mpRenderSurface(0)
+		, mpRenderZBuffer(0)
 	{
 		mIsFullScreen = false;
 		mIsSwapChain = (deviceIfSwapChain != NULL);
@@ -141,7 +140,11 @@ namespace Ogre
 			// FSAA type
 			opt = miscParams->find("FSAA");
 			if(opt != miscParams->end())
-				mFSAAType = (D3DMULTISAMPLE_TYPE)StringConverter::parseUnsignedInt(opt->second);
+			{
+				mFSAA = StringConverter::parseUnsignedInt(opt->second);
+				mFSAAType = (D3DMULTISAMPLE_TYPE)mFSAA;
+			}
+
 			// FSAA quality
 			opt = miscParams->find("FSAAQuality");
 			if(opt != miscParams->end())
@@ -158,7 +161,12 @@ namespace Ogre
 			opt = miscParams->find("useNVPerfHUD");
 			if(opt != miscParams->end())
 				mUseNVPerfHUD = StringConverter::parseBool(opt->second);
-			 
+			// sRGB?
+			opt = miscParams->find("gamma");
+			if(opt != miscParams->end())
+				mHwGamma = StringConverter::parseBool(opt->second);
+
+
 		}
 
 		// Destroy current window if any
@@ -232,7 +240,7 @@ namespace Ogre
 			mIsExternal = false;
 			mHWnd = CreateWindow("OgreD3D9Wnd", title.c_str(), dwStyle,
 				mLeft, mTop, mWidth, mHeight, parentHWnd, 0, hInst, this);
-			
+
 			WindowEventUtilities::_addRenderWindow(this);
 		}
 		else
@@ -256,12 +264,10 @@ namespace Ogre
 		mIsFullScreen = fullScreen;
 		mColourDepth = colourDepth;
 
-		StringUtil::StrStreamType str;
-		str << "D3D9 : Created D3D9 Rendering Window '"
+		LogManager::getSingleton().stream()
+			<< "D3D9 : Created D3D9 Rendering Window '"
 			<< mName << "' : " << mWidth << "x" << mHeight 
 			<< ", " << mColourDepth << "bpp";
-		LogManager::getSingleton().logMessage(
-			LML_NORMAL, str.str());
 
 		createD3DResources();
 
@@ -398,7 +404,7 @@ namespace Ogre
 		md3dpp.BackBufferWidth			= mWidth;
 		md3dpp.BackBufferHeight			= mHeight;
 		md3dpp.FullScreen_RefreshRateInHz = mIsFullScreen ? mDisplayFrequency : 0;
-		
+
 
 		if (mVSync)
 		{
@@ -460,6 +466,22 @@ namespace Ogre
 		md3dpp.MultiSampleType = mFSAAType;
 		md3dpp.MultiSampleQuality = (mFSAAQuality == 0) ? 0 : mFSAAQuality;
 
+		// Check sRGB
+		if (mHwGamma)
+		{
+			/* hmm, this never succeeds even when device does support??
+			if(FAILED(pD3D->CheckDeviceFormat(mDriver->getAdapterNumber(),
+			devType, md3dpp.BackBufferFormat, D3DUSAGE_QUERY_SRGBWRITE, 
+			D3DRTYPE_SURFACE, md3dpp.BackBufferFormat )))
+			{
+			// disable - not supported
+			mHwGamma = false;
+			}
+			*/
+
+		}
+
+
 		if (mIsSwapChain)
 		{
 			// Create swap chain			
@@ -479,11 +501,13 @@ namespace Ogre
 					"D3D9RenderWindow::createD3DResources");
 			}
 			// Store references to buffers for convenience
+			SAFE_RELEASE(mpRenderSurface);
 			mpSwapChain->GetBackBuffer( 0, D3DBACKBUFFER_TYPE_MONO, &mpRenderSurface );
 			// Additional swap chains need their own depth buffer
 			// to support resizing them
 			if (mIsDepthBuffered) 
 			{
+				SAFE_RELEASE(mpRenderZBuffer);
 				hr = mpD3DDevice->CreateDepthStencilSurface(
 					mWidth, mHeight,
 					md3dpp.AutoDepthStencilFormat,
@@ -519,7 +543,7 @@ namespace Ogre
 				if (opti != options.end() && opti->second.currentValue == "Consistent")
 					extraFlags |= D3DCREATE_FPU_PRESERVE;
 
-#if OGRE_THREAD_SUPPORT
+#if OGRE_THREAD_SUPPORT == 1
 				extraFlags |= D3DCREATE_MULTITHREADED;
 #endif
 				// Set default settings (use the one Ogre discovered as a default)
@@ -576,7 +600,9 @@ namespace Ogre
 			// update device in driver
 			mDriver->setD3DDevice( mpD3DDevice );
 			// Store references to buffers for convenience
+			SAFE_RELEASE(mpRenderSurface);
 			mpD3DDevice->GetRenderTarget( 0, &mpRenderSurface );
+			SAFE_RELEASE(mpRenderZBuffer);
 			mpD3DDevice->GetDepthStencilSurface( &mpRenderZBuffer );
 			// release immediately so we don't hog them
 			mpRenderZBuffer->Release();
@@ -677,10 +703,9 @@ namespace Ogre
 
 			if (FAILED(hr)) 
 			{
-				StringUtil::StrStreamType str;
-				str << "D3D9RenderWindow: failed to reset device to new dimensions << "
+				LogManager::getSingleton().stream(LML_CRITICAL)
+					<< "D3D9RenderWindow: failed to reset device to new dimensions << "
 					<< width << " x " << height << ". Trying to recover.";
-				LogManager::getSingleton().logMessage(LML_CRITICAL, str.str());
 
 				// try to recover
 				hr = mDriver->getD3DDevice()->CreateAdditionalSwapChain(
@@ -698,7 +723,9 @@ namespace Ogre
 				mWidth = width;
 				mHeight = height;
 
+				SAFE_RELEASE(mpRenderSurface);
 				hr = mpSwapChain->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &mpRenderSurface);
+				SAFE_RELEASE(mpRenderZBuffer);
 				hr = mDriver->getD3DDevice()->CreateDepthStencilSurface(
 					mWidth, mHeight,
 					md3dpp.AutoDepthStencilFormat,
@@ -802,153 +829,232 @@ namespace Ogre
 		}
 	}
 
-	void D3D9RenderWindow::writeContentsToFile(const String& filename)
+	void D3D9RenderWindow::copyContentsToMemory(const PixelBox &dst, FrameBuffer buffer)
 	{
+		if ((dst.left < 0) || (dst.right > mWidth) ||
+			(dst.top < 0) || (dst.bottom > mHeight) ||
+			(dst.front != 0) || (dst.back != 1))
+		{
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+				"Invalid box.",
+				"D3D9RenderWindow::copyContentsToMemory" );
+		}
+
 		HRESULT hr;
-		LPDIRECT3DSURFACE9 pSurf=NULL, pTempSurf=NULL;
+		LPDIRECT3DSURFACE9 pSurf = 0, pTempSurf = 0;
 		D3DSURFACE_DESC desc;
-		D3DDISPLAYMODE dm;
+		D3DLOCKED_RECT lockedRect;
 
-		// access device through driver
 		LPDIRECT3DDEVICE9 mpD3DDevice = mDriver->getD3DDevice();
 
-		// get display dimensions
-		// this will be the dimensions of the front buffer
-		if (FAILED(hr = mpD3DDevice->GetDisplayMode(0, &dm)))
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get display mode!", "D3D9RenderWindow::writeContentsToFile");
-
-		desc.Width = dm.Width;
-		desc.Height = dm.Height;
-		desc.Format = D3DFMT_A8R8G8B8;
-		if (FAILED(hr = mpD3DDevice->CreateOffscreenPlainSurface(
-			desc.Width, 
-			desc.Height, 
-			desc.Format, 
-			D3DPOOL_SYSTEMMEM, 
-			&pTempSurf, 
-			NULL)))
+		if (buffer == FB_AUTO)
 		{
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Cannot create offscreen buffer 1!", "D3D9RenderWindow::writeContentsToFile");
+			//buffer = mIsFullScreen? FB_FRONT : FB_BACK;
+			buffer = FB_FRONT;
 		}
 
-		if (FAILED(hr = mpD3DDevice->GetFrontBufferData(0, pTempSurf)))
+		if (buffer == FB_FRONT)
 		{
-			SAFE_RELEASE(pTempSurf);
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get front buffer!", "D3D9RenderWindow::writeContentsToFile");
-		}
-		
-		D3DLOCKED_RECT lockedRect;
-		if(mIsFullScreen)
-		{
-			if (FAILED(hr = pTempSurf->LockRect(&lockedRect, NULL, 
-			D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK)))
+			D3DDISPLAYMODE dm;
+
+			LPDIRECT3DDEVICE9 mpD3DDevice = mDriver->getD3DDevice();
+
+			if (FAILED(hr = mpD3DDevice->GetDisplayMode(0, &dm)))
 			{
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "can't lock rect!", "D3D9RenderWindow::writeContentsToFile");
-			} 
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+					"Can't get display mode: " + Root::getSingleton().getErrorDescription(hr),
+					"D3D9RenderWindow::copyContentsToMemory");
+			}
+
+			desc.Width = dm.Width;
+			desc.Height = dm.Height;
+			desc.Format = D3DFMT_A8R8G8B8;
+			if (FAILED(hr = mpD3DDevice->CreateOffscreenPlainSurface(desc.Width, desc.Height,
+				desc.Format,
+				D3DPOOL_SYSTEMMEM,
+				&pTempSurf,
+				0)))
+			{
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+					"Can't create offscreen buffer: " + Root::getSingleton().getErrorDescription(hr),
+					"D3D9RenderWindow::copyContentsToMemory");
+			}
+
+			if (FAILED(hr = mpD3DDevice->GetFrontBufferData(0, pTempSurf)))
+			{
+				SAFE_RELEASE(pTempSurf);
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+					"Can't get front buffer: " + Root::getSingleton().getErrorDescription(hr),
+					"D3D9RenderWindow::copyContentsToMemory");
+			}
+
+			if(mIsFullScreen)
+			{
+				if ((dst.left == 0) && (dst.right == mWidth) && (dst.top == 0) && (dst.bottom == mHeight))
+				{
+					hr = pTempSurf->LockRect(&lockedRect, 0, D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK);
+				}
+				else
+				{
+					RECT rect;
+
+					rect.left = (LONG)dst.left;
+					rect.right = (LONG)dst.right;
+					rect.top = (LONG)dst.top;
+					rect.bottom = (LONG)dst.bottom;
+
+					hr = pTempSurf->LockRect(&lockedRect, &rect, D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK);
+				}
+				if (FAILED(hr))
+				{
+					SAFE_RELEASE(pTempSurf);
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+						"Can't lock rect: " + Root::getSingleton().getErrorDescription(hr),
+						"D3D9RenderWindow::copyContentsToMemory");
+				} 
+			}
+			else
+			{
+				RECT srcRect;
+				//GetClientRect(mHWnd, &srcRect);
+				srcRect.left = (LONG)dst.left;
+				srcRect.top = (LONG)dst.top;
+				srcRect.right = (LONG)dst.right;
+				srcRect.bottom = (LONG)dst.bottom;
+				POINT point;
+				point.x = srcRect.left;
+				point.y = srcRect.top;
+				ClientToScreen(mHWnd, &point);
+				srcRect.top = point.y;
+				srcRect.left = point.x;
+				srcRect.bottom += point.y;
+				srcRect.right += point.x;
+
+				desc.Width = srcRect.right - srcRect.left;
+				desc.Height = srcRect.bottom - srcRect.top;
+
+				if (FAILED(hr = pTempSurf->LockRect(&lockedRect, &srcRect, D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK)))
+				{
+					SAFE_RELEASE(pTempSurf);
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+						"Can't lock rect: " + Root::getSingleton().getErrorDescription(hr),
+						"D3D9RenderWindow::copyContentsToMemory");
+				} 
+			}
 		}
 		else
 		{
-			RECT srcRect;
-			GetClientRect(mHWnd, &srcRect);
-			POINT point;
-			point.x = srcRect.left;
-			point.y = srcRect.top;
-			ClientToScreen(mHWnd, &point);
-			srcRect.top = point.y;
-			srcRect.left = point.x;
-			srcRect.bottom += point.y;
-			srcRect.right += point.x;
-
-			desc.Width = srcRect.right - srcRect.left;
-			desc.Height = srcRect.bottom - srcRect.top;
-
-			if (FAILED(hr = pTempSurf->LockRect(&lockedRect, &srcRect, 
-
-			D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK)))
+			SAFE_RELEASE(pSurf);
+			if(FAILED(hr = mpD3DDevice->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pSurf)))
 			{
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "can't lock rect!", "D3D9RenderWindow::writeContentsToFile");
-			} 
-		}
-  
-		ImageCodec::ImageData *imgData = new ImageCodec::ImageData();
-		imgData->width = desc.Width;
-		imgData->height = desc.Height;
-		imgData->format = PF_BYTE_RGB;
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+					"Can't get back buffer: " + Root::getSingleton().getErrorDescription(hr),
+					"D3D9RenderWindow::copyContentsToMemory");
+			}
 
-		// Allocate contiguous buffer (surfaces aren't necessarily contiguous)
-		uchar* pBuffer = new uchar[desc.Width * desc.Height * 3];
-
-		uint x, y;
-		uchar *pData, *pDest;
-
-		pData = (uchar*)lockedRect.pBits;
-		pDest = pBuffer;
-		for (y = 0; y < desc.Height; ++y)
-		{
-			uchar *pRow = pData;
-
-			for (x = 0; x < desc.Width; ++x)
+			if(FAILED(hr = pSurf->GetDesc(&desc)))
 			{
-				switch(desc.Format)
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+					"Can't get description: " + Root::getSingleton().getErrorDescription(hr),
+					"D3D9RenderWindow::copyContentsToMemory");
+			}
+
+			if (FAILED(hr = mpD3DDevice->CreateOffscreenPlainSurface(desc.Width, desc.Height,
+				desc.Format,
+				D3DPOOL_SYSTEMMEM,
+				&pTempSurf,
+				0)))
+			{
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+					"Can't create offscreen surface: " + Root::getSingleton().getErrorDescription(hr),
+					"D3D9RenderWindow::copyContentsToMemory");
+			}
+
+			if (desc.MultiSampleType == D3DMULTISAMPLE_NONE)
+			{
+				if (FAILED(hr = mpD3DDevice->GetRenderTargetData(pSurf, pTempSurf)))
 				{
-				case D3DFMT_R5G6B5:
-					WORD val;
+					SAFE_RELEASE(pTempSurf);
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+						"Can't get render target data: " + Root::getSingleton().getErrorDescription(hr),
+						"D3D9RenderWindow::copyContentsToMemory");
+				}
+			}
+			else
+			{
+				LPDIRECT3DSURFACE9 pStretchSurf = 0;
 
-					val = *((WORD*)pRow);
-					pRow += 2;
-
-					*pDest++ = Bitwise::convertBitPattern((WORD)val, (WORD)0xF800, (BYTE)0xFF);
-					*pDest++ = Bitwise::convertBitPattern((WORD)val, (WORD)0x07E0, (BYTE)0xFF);
-					*pDest++ = Bitwise::convertBitPattern((WORD)val, (WORD)0x001F, (BYTE)0xFF);
-					break;
-				case D3DFMT_A8R8G8B8:
-				case D3DFMT_X8R8G8B8:
-					// Actual format is BRGA for some reason
-					*pDest++ = pRow[2]; // R
-					*pDest++ = pRow[1]; // G
-					*pDest++ = pRow[0]; // B
-					pRow += 4; // skip alpha / dummy
-					break;
-				case D3DFMT_R8G8B8:
-					// Actual format is BRGA for some reason
-					*pDest++ = pRow[2]; // R
-					*pDest++ = pRow[1]; // G
-					*pDest++ = pRow[0]; // B
-					pRow += 3; 
-					break;
+				if (FAILED(hr = mpD3DDevice->CreateRenderTarget(desc.Width, desc.Height,
+					desc.Format,
+					D3DMULTISAMPLE_NONE,
+					0,
+					false,
+					&pStretchSurf,
+					0)))
+				{
+					SAFE_RELEASE(pTempSurf);
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+						"Can't create render target: " + Root::getSingleton().getErrorDescription(hr),
+						"D3D9RenderWindow::copyContentsToMemory");
 				}
 
+				if (FAILED(hr = mpD3DDevice->StretchRect(pSurf, 0, pStretchSurf, 0, D3DTEXF_NONE)))
+				{
+					SAFE_RELEASE(pTempSurf);
+					SAFE_RELEASE(pStretchSurf);
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+						"Can't stretch rect: " + Root::getSingleton().getErrorDescription(hr),
+						"D3D9RenderWindow::copyContentsToMemory");
+				}
+				if (FAILED(hr = mpD3DDevice->GetRenderTargetData(pStretchSurf, pTempSurf)))
+				{
+					SAFE_RELEASE(pTempSurf);
+					SAFE_RELEASE(pStretchSurf);
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+						"Can't get render target data: " + Root::getSingleton().getErrorDescription(hr),
+						"D3D9RenderWindow::copyContentsToMemory");
+				}
+				SAFE_RELEASE(pStretchSurf);
+			}
 
+			if ((dst.left == 0) && (dst.right == mWidth) && (dst.top == 0) && (dst.bottom == mHeight))
+			{
+				hr = pTempSurf->LockRect(&lockedRect, 0, D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK);
+			}
+			else
+			{
+				RECT rect;
+
+				rect.left = (LONG)dst.left;
+				rect.right = (LONG)dst.right;
+				rect.top = (LONG)dst.top;
+				rect.bottom = (LONG)dst.bottom;
+
+				hr = pTempSurf->LockRect(&lockedRect, &rect, D3DLOCK_READONLY | D3DLOCK_NOSYSLOCK);
+			}
+			if (FAILED(hr))
+			{
+				SAFE_RELEASE(pTempSurf);
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+					"Can't lock rect: " + Root::getSingleton().getErrorDescription(hr),
+					"D3D9RenderWindow::copyContentsToMemory");
 			}
-			// increase by one line
-			pData += lockedRect.Pitch;
 		}
 
-		// Wrap buffer in a chunk
-		MemoryDataStreamPtr stream(new MemoryDataStream(pBuffer, desc.Width * desc.Height * 3, false));
+		PixelFormat format = Ogre::D3D9Mappings::_getPF(desc.Format);
 
-		// Get codec 
-		size_t pos = filename.find_last_of(".");
-		String extension;
-		if( pos == String::npos )
-			OGRE_EXCEPT(
-			Exception::ERR_INVALIDPARAMS, 
-			"Unable to determine image type for '" + filename + "' - invalid extension.",
-			"D3D9RenderWindow::writeContentsToFile" );
-
-		while( pos != filename.length() - 1 )
-			extension += filename[++pos];
-
-		// Get the codec
-		Codec * pCodec = Codec::getCodec(extension);
-
-		// Write out
+		if (format == PF_UNKNOWN)
 		{
-			Codec::CodecDataPtr ptr(imgData);
-			pCodec->codeToFile(stream, filename, ptr);
+			SAFE_RELEASE(pTempSurf);
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+				"Unsupported format", "D3D9RenderWindow::copyContentsToMemory");
 		}
 
-		delete [] pBuffer;
+		PixelBox src(dst.getWidth(), dst.getHeight(), 1, format, lockedRect.pBits);
+		src.rowPitch = lockedRect.Pitch / PixelUtil::getNumElemBytes(format);
+		src.slicePitch = desc.Height * src.rowPitch;
+
+		PixelUtil::bulkPixelConversion(src, dst);
 
 		SAFE_RELEASE(pTempSurf);
 		SAFE_RELEASE(pSurf);
@@ -998,17 +1104,19 @@ namespace Ogre
 				if (!mIsSwapChain) 
 				{
 					// re-qeuery buffers
+					SAFE_RELEASE(mpRenderSurface);
 					mpD3DDevice->GetRenderTarget( 0, &mpRenderSurface );
+					SAFE_RELEASE(mpRenderZBuffer);
 					mpD3DDevice->GetDepthStencilSurface( &mpRenderZBuffer );
 					// release immediately so we don't hog them
 					mpRenderZBuffer->Release();
 				}
 				else 
 				{
-				    // Update dimensions incase changed
-		            ViewportList::iterator it = mViewportList.begin();
-		            while( it != mViewportList.end() )
-			            (*it++).second->_updateDimensions();
+					// Update dimensions incase changed
+					ViewportList::iterator it = mViewportList.begin();
+					while( it != mViewportList.end() )
+						(*it++).second->_updateDimensions();
 					// Actual restoration of surfaces will happen in 
 					// D3D9RenderSystem::restoreLostDevice when it calls
 					// createD3DResources for each secondary window
