@@ -34,12 +34,11 @@ Torus Knot Software Ltd.
 #include "OgreBitwise.h"
 #include "OgreD3D9Mappings.h"
 #include "OgreD3D9RenderSystem.h"
+#include "OgreD3D9TextureManager.h"
 #include "OgreRoot.h"
 
-#include "OgreNoMemoryMacros.h"
 #include <d3dx9.h>
 #include <dxerr9.h>
-#include "OgreMemoryMacros.h"
 
 namespace Ogre 
 {
@@ -53,9 +52,13 @@ namespace Ogre
         mpNormTex(NULL),
         mpCubeTex(NULL),
 		mpVolumeTex(NULL),
+		mFSAASurface(NULL),
         mpTex(NULL),
         mD3DPool(D3DPOOL_MANAGED),
-		mDynamicTextures(false)
+		mDynamicTextures(false),
+		mHwGammaReadSupported(false),
+		mHwGammaWriteSupported(false),
+		mFSAALevelSupported(false)
 	{
         _initDevice();
 	}
@@ -72,6 +75,8 @@ namespace Ogre
 		{
 			freeInternalResources();
 		}
+
+		static_cast<D3D9TextureManager*>(mCreator)->_notifyDestroyed(this);
 	}
 	/****************************************************************************************/
 	void D3D9Texture::copyToTexture(TexturePtr& target)
@@ -189,23 +194,139 @@ namespace Ogre
             mD3DPool = D3DPOOL_MANAGED;
         }
 
+        // only copy is on the stack so well-behaved if exception thrown
+        LoadedStreams loadedStreams = mLoadedStreams;
+        mLoadedStreams.setNull();
+
 		// load based on tex.type
 		switch (this->getTextureType())
 		{
 		case TEX_TYPE_1D:
 		case TEX_TYPE_2D:
-			this->_loadNormTex();
+			this->_loadNormTex(loadedStreams);
 			break;
 		case TEX_TYPE_3D:
-            this->_loadVolumeTex();
+            this->_loadVolumeTex(loadedStreams);
             break;
 		case TEX_TYPE_CUBE_MAP:
-			this->_loadCubeTex();
+			this->_loadCubeTex(loadedStreams);
 			break;
 		default:
 			OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Unknown texture type", "D3D9Texture::loadImpl" );
 		}
 
+	}
+	/****************************************************************************************/
+	void D3D9Texture::prepareImpl()
+	{
+		if (mUsage & TU_RENDERTARGET)
+		{
+			return;
+		}
+
+        LoadedStreams loadedStreams;
+
+		// prepare load based on tex.type
+		switch (this->getTextureType())
+		{
+		case TEX_TYPE_1D:
+		case TEX_TYPE_2D:
+			loadedStreams = this->_prepareNormTex();
+			break;
+		case TEX_TYPE_3D:
+            loadedStreams = this->_prepareVolumeTex();
+            break;
+		case TEX_TYPE_CUBE_MAP:
+			loadedStreams = this->_prepareCubeTex();
+			break;
+		default:
+			OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Unknown texture type", "D3D9Texture::prepareImpl" );
+		}
+
+        mLoadedStreams = loadedStreams;
+	}
+    /****************************************************************************************/
+	D3D9Texture::LoadedStreams D3D9Texture::_prepareCubeTex()
+	{
+		assert(this->getTextureType() == TEX_TYPE_CUBE_MAP);
+
+        LoadedStreams loadedStreams = LoadedStreams(new std::vector<MemoryDataStreamPtr>());
+        // DDS load?
+		if (getSourceFileType() == "dds")
+		{
+            // find & load resource data
+			DataStreamPtr dstream = 
+				ResourceGroupManager::getSingleton().openResource(
+					mName, mGroup, true, this);
+            loadedStreams->push_back(MemoryDataStreamPtr(new MemoryDataStream(dstream)));
+        }
+        else
+        {
+			// Load from 6 separate files
+			// Use OGRE its own codecs
+			String baseName, ext;
+			size_t pos = mName.find_last_of(".");
+			baseName = mName.substr(0, pos);
+			if ( pos != String::npos )
+				ext = mName.substr(pos+1);
+			static const String suffixes[6] = {"_rt", "_lf", "_up", "_dn", "_fr", "_bk"};
+
+			for(size_t i = 0; i < 6; i++)
+			{
+				String fullName = baseName + suffixes[i];
+				if (!ext.empty())
+					fullName = fullName + "." + ext;
+
+            	// find & load resource data intro stream to allow resource
+				// group changes if required
+				DataStreamPtr dstream = 
+					ResourceGroupManager::getSingleton().openResource(
+						fullName, mGroup, true, this);
+
+                loadedStreams->push_back(MemoryDataStreamPtr(new MemoryDataStream(dstream)));
+			}
+        }
+
+        return loadedStreams;
+	}
+	/****************************************************************************************/
+	D3D9Texture::LoadedStreams D3D9Texture::_prepareVolumeTex()
+	{
+		assert(this->getTextureType() == TEX_TYPE_3D);
+
+		// find & load resource data
+		DataStreamPtr dstream = 
+			ResourceGroupManager::getSingleton().openResource(
+				mName, mGroup, true, this);
+
+        LoadedStreams loadedStreams = LoadedStreams(new std::vector<MemoryDataStreamPtr>());
+        loadedStreams->push_back(MemoryDataStreamPtr(new MemoryDataStream(dstream)));
+        return loadedStreams;
+    }
+	/****************************************************************************************/
+	D3D9Texture::LoadedStreams D3D9Texture::_prepareNormTex()
+	{
+		assert(this->getTextureType() == TEX_TYPE_1D || this->getTextureType() == TEX_TYPE_2D);
+
+		// find & load resource data
+		DataStreamPtr dstream = 
+			ResourceGroupManager::getSingleton().openResource(
+				mName, mGroup, true, this);
+
+        LoadedStreams loadedStreams = LoadedStreams(new std::vector<MemoryDataStreamPtr>());
+        loadedStreams->push_back(MemoryDataStreamPtr(new MemoryDataStream(dstream)));
+        return loadedStreams;
+	}
+	/****************************************************************************************/
+	void D3D9Texture::unprepareImpl()
+	{
+		if (mUsage & TU_RENDERTARGET)
+		{
+			return;
+		}
+
+        mLoadedStreams.setNull();
+
 	}
 	/****************************************************************************************/
 	void D3D9Texture::freeInternalResourcesImpl()
@@ -214,20 +335,18 @@ namespace Ogre
 		SAFE_RELEASE(mpNormTex);
 		SAFE_RELEASE(mpCubeTex);
 		SAFE_RELEASE(mpVolumeTex);
+		SAFE_RELEASE(mFSAASurface);
 	}
 	/****************************************************************************************/
-	void D3D9Texture::_loadCubeTex()
+	void D3D9Texture::_loadCubeTex(const D3D9Texture::LoadedStreams &loadedStreams)
 	{
 		assert(this->getTextureType() == TEX_TYPE_CUBE_MAP);
 
         // DDS load?
-		if (StringUtil::endsWith(getName(), ".dds"))
-        {
+		if (getSourceFileType() == "dds")
+		{
             // find & load resource data
-			DataStreamPtr dstream = 
-				ResourceGroupManager::getSingleton().openResource(
-					mName, mGroup, true, this);
-            MemoryDataStream stream( dstream );
+            assert(loadedStreams->size()==1);
 
 			DWORD usage = 0;
 			UINT numMips = (mNumRequestedMipmaps == MIP_UNLIMITED) ?
@@ -242,7 +361,7 @@ namespace Ogre
 
             // Determine D3D pool to use
             D3DPOOL pool;
-            if (mUsage & TU_DYNAMIC)
+            if (useDefaultPool())
             {
                 pool = D3DPOOL_DEFAULT;
             }
@@ -253,8 +372,8 @@ namespace Ogre
 
 			HRESULT hr = D3DXCreateCubeTextureFromFileInMemoryEx(
 				mpDev,
-				stream.getPtr(),
-				stream.size(),
+				(*loadedStreams)[0]->getPtr(),
+				(*loadedStreams)[0]->size(),
 				D3DX_DEFAULT, // dims (square)
 				numMips,
 				usage,
@@ -270,7 +389,8 @@ namespace Ogre
             if (FAILED(hr))
 		    {
 				this->freeInternalResources();
-			    OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't create cube texture", "D3D9Texture::_loadCubeTex" );
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't create cube texture: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_loadCubeTex" );
 		    }
 
             hr = mpCubeTex->QueryInterface(IID_IDirect3DBaseTexture9, (void **)&mpTex);
@@ -278,7 +398,8 @@ namespace Ogre
             if (FAILED(hr))
 		    {
 				this->freeInternalResources();
-			    OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture", "D3D9Texture::_loadCubeTex" );
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_loadCubeTex" );
 		    }
 
             D3DSURFACE_DESC texDesc;
@@ -291,27 +412,20 @@ namespace Ogre
         }
         else
         {
-			// Load from 6 separate files
-			// Use OGRE its own codecs
-			String baseName, ext;
+            assert(loadedStreams->size()==6);
+
+			String  ext;
 			size_t pos = mName.find_last_of(".");
-			baseName = mName.substr(0, pos);
-			ext = mName.substr(pos+1);
+			if ( pos != String::npos )
+				ext = mName.substr(pos+1);
+
 			std::vector<Image> images(6);
 			ConstImagePtrList imagePtrs;
-			static const String suffixes[6] = {"_rt", "_lf", "_up", "_dn", "_fr", "_bk"};
 
 			for(size_t i = 0; i < 6; i++)
 			{
-				String fullName = baseName + suffixes[i] + "." + ext;
-
-            	// find & load resource data intro stream to allow resource
-				// group changes if required
-				DataStreamPtr dstream = 
-					ResourceGroupManager::getSingleton().openResource(
-						fullName, mGroup, true, this);
-	
-				images[i].load(dstream, ext);
+				DataStreamPtr stream((*loadedStreams)[i]);
+				images[i].load(stream, ext);
 
 				imagePtrs.push_back(&images[i]);
 			}
@@ -320,17 +434,14 @@ namespace Ogre
         }
 	}
 	/****************************************************************************************/
-	void D3D9Texture::_loadVolumeTex()
+	void D3D9Texture::_loadVolumeTex(const D3D9Texture::LoadedStreams &loadedStreams)
 	{
 		assert(this->getTextureType() == TEX_TYPE_3D);
 		// DDS load?
-		if (StringUtil::endsWith(getName(), ".dds"))
+		if (getSourceFileType() == "dds")
 		{
 			// find & load resource data
-			DataStreamPtr dstream = 
-				ResourceGroupManager::getSingleton().openResource(
-					mName, mGroup, true, this);
-			MemoryDataStream stream(dstream);
+            assert(loadedStreams->size()==1);
 	
 			DWORD usage = 0;
 			UINT numMips = (mNumRequestedMipmaps == MIP_UNLIMITED) ?
@@ -345,7 +456,7 @@ namespace Ogre
 
             // Determine D3D pool to use
             D3DPOOL pool;
-            if (mUsage & TU_DYNAMIC)
+            if (useDefaultPool())
             {
                 pool = D3DPOOL_DEFAULT;
             }
@@ -356,8 +467,8 @@ namespace Ogre
 
 			HRESULT hr = D3DXCreateVolumeTextureFromFileInMemoryEx(
 				mpDev,
-				stream.getPtr(),
-				stream.size(),
+				(*loadedStreams)[0]->getPtr(),
+				(*loadedStreams)[0]->size(),
 				D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, // dims
 				numMips,
 				usage,
@@ -373,7 +484,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, 
-					"Unable to load volume texture from " + this->getName(),
+					"Unable to load volume texture from " + this->getName() + ": " + String(DXGetErrorDescription9(hr)),
 					"D3D9Texture::_loadVolumeTex");
 			}
 	
@@ -382,7 +493,8 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				this->freeInternalResources();
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture", "D3D9Texture::_loadVolumeTex" );
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_loadVolumeTex" );
 			}
 	
 			D3DVOLUME_DESC texDesc;
@@ -396,15 +508,16 @@ namespace Ogre
 		else
 		{
 			Image img;
-           	// find & load resource data intro stream to allow resource
-			// group changes if required
-			DataStreamPtr dstream = 
-				ResourceGroupManager::getSingleton().openResource(
-					mName, mGroup, true, this);
+
+            assert(loadedStreams->size()==1);
+
 			size_t pos = mName.find_last_of(".");
-			String ext = mName.substr(pos+1);
+			String ext;
+			if ( pos != String::npos )
+				ext = mName.substr(pos+1);
 	
-			img.load(dstream, ext);
+			DataStreamPtr stream((*loadedStreams)[0]);
+			img.load(stream, ext);
 			// Call internal _loadImages, not loadImage since that's external and 
 			// will determine load status etc again
 			ConstImagePtrList imagePtrs;
@@ -413,18 +526,14 @@ namespace Ogre
 		}
     }
 	/****************************************************************************************/
-	void D3D9Texture::_loadNormTex()
+	void D3D9Texture::_loadNormTex(const D3D9Texture::LoadedStreams &loadedStreams)
 	{
 		assert(this->getTextureType() == TEX_TYPE_1D || this->getTextureType() == TEX_TYPE_2D);
 		// DDS load?
-		if (StringUtil::endsWith(getName(), ".dds"))
+		if (getSourceFileType() == "dds")
 		{
 			// Use D3DX
-			// find & load resource data
-			DataStreamPtr dstream = 
-				ResourceGroupManager::getSingleton().openResource(
-					mName, mGroup, true, this);
-			MemoryDataStream stream(dstream);
+            assert(loadedStreams->size()==1);
 	
 			DWORD usage = 0;
 			UINT numMips = (mNumRequestedMipmaps == MIP_UNLIMITED) ?
@@ -439,7 +548,7 @@ namespace Ogre
 
             // Determine D3D pool to use
             D3DPOOL pool;
-            if (mUsage & TU_DYNAMIC)
+            if (useDefaultPool())
             {
                 pool = D3DPOOL_DEFAULT;
             }
@@ -450,8 +559,8 @@ namespace Ogre
 
 			HRESULT hr = D3DXCreateTextureFromFileInMemoryEx(
 				mpDev,
-				stream.getPtr(),
-				stream.size(),
+				(*loadedStreams)[0]->getPtr(),
+				(*loadedStreams)[0]->size(),
 				D3DX_DEFAULT, D3DX_DEFAULT, // dims
 				numMips,
 				usage,
@@ -467,7 +576,7 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, 
-					"Unable to load texture from " + this->getName(),
+					"Unable to load texture from " + this->getName() + " :" + String(DXGetErrorDescription9(hr)),
 					"D3D9Texture::_loadNormTex");
 			}
 	
@@ -476,7 +585,8 @@ namespace Ogre
 			if (FAILED(hr))
 			{
 				this->freeInternalResources();
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture", "D3D9Texture::_loadNormTex" );
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_loadNormTex" );
 			}
 	
 			D3DSURFACE_DESC texDesc;
@@ -492,20 +602,22 @@ namespace Ogre
 			Image img;
            	// find & load resource data intro stream to allow resource
 			// group changes if required
-			DataStreamPtr dstream = 
-				ResourceGroupManager::getSingleton().openResource(
-					mName, mGroup, true, this);
+            assert(loadedStreams->size()==1);
 	
 			size_t pos = mName.find_last_of(".");
-			String ext = mName.substr(pos+1);
-			
-			img.load(dstream, ext);
+			String ext; 
+			if ( pos != String::npos )
+				ext = mName.substr(pos+1);
+
+			DataStreamPtr stream((*loadedStreams)[0]);
+			img.load(stream, ext);
 			// Call internal _loadImages, not loadImage since that's external and 
 			// will determine load status etc again
 			ConstImagePtrList imagePtrs;
 			imagePtrs.push_back(&img);
 			_loadImages( imagePtrs );
 		}
+
 	}
 	/****************************************************************************************/
     void D3D9Texture::createInternalResourcesImpl(void)
@@ -520,8 +632,7 @@ namespace Ogre
 		// Determine D3D pool to use
 		// Use managed unless we're a render target or user has asked for 
 		// a dynamic texture
-		if ((mUsage & TU_RENDERTARGET) ||
-			(mUsage & TU_DYNAMIC))
+		if (useDefaultPool())
 		{
 			mD3DPool = D3DPOOL_DEFAULT;
 		}
@@ -576,6 +687,18 @@ namespace Ogre
 				mDynamicTextures = false;
 			}
 		}
+		// Check sRGB support
+		if (mHwGamma)
+		{
+			mHwGammaReadSupported = _canUseHardwareGammaCorrection(usage, D3DRTYPE_TEXTURE, d3dPF, false);
+			if (mUsage & TU_RENDERTARGET)
+				mHwGammaWriteSupported = _canUseHardwareGammaCorrection(usage, D3DRTYPE_TEXTURE, d3dPF, true);
+		}
+		// Check FSAA level
+		if (mFSAA > 0 && (mUsage & TU_RENDERTARGET))
+		{
+			mFSAALevelSupported = _canUseFSAALevel(usage, D3DRTYPE_TEXTURE, d3dPF, mFSAA);
+		}
 		// check if mip maps are supported on hardware
 		mMipmapsHardwareGenerated = false;
 		if (mDevCaps.TextureCaps & D3DPTEXTURECAPS_MIPMAP)
@@ -612,7 +735,8 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture", "D3D9Texture::_createNormTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createNormTex" );
 		}
 		
 		// set the base texture we'll use in the render system
@@ -620,7 +744,8 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture", "D3D9Texture::_createNormTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createNormTex" );
 		}
 		
 		// set final tex. attributes from tex. description
@@ -630,8 +755,28 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description", "D3D9Texture::_createNormTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createNormTex" );
 		}
+
+		if (mFSAA > 0 && mFSAALevelSupported)
+		{
+			// create AA surface
+			HRESULT hr = mpDev->CreateRenderTarget(desc.Width, desc.Height, d3dPF, 
+				(D3DMULTISAMPLE_TYPE)mFSAA, 
+				0, // Only supporting regular sampling, not nonmaskable 
+				FALSE, // not lockable
+				&mFSAASurface, NULL);
+
+			if (FAILED(hr))
+			{
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
+					"Unable to create AA render target: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_createNormTex");
+			}
+
+		}
+
 		this->_setFinalAttributes(desc.Width, desc.Height, 1, D3D9Mappings::_getPF(desc.Format));
 		
 		// Set best filter type
@@ -640,9 +785,11 @@ namespace Ogre
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
 			{
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type", "D3D9Texture::_createNormTex" );
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: "  + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_createNormTex" );
 			}
 		}
+
 	}
 	/****************************************************************************************/
 	void D3D9Texture::_createCubeTex()
@@ -673,6 +820,18 @@ namespace Ogre
 				mDynamicTextures = false;
 			}
 		}
+		// Check sRGB support
+		if (mHwGamma)
+		{
+			mHwGammaReadSupported = _canUseHardwareGammaCorrection(usage, D3DRTYPE_CUBETEXTURE, d3dPF, false);
+			if (mUsage & TU_RENDERTARGET)
+				mHwGammaWriteSupported = _canUseHardwareGammaCorrection(usage, D3DRTYPE_CUBETEXTURE, d3dPF, true);
+		}
+		// Check FSAA level
+		if (mFSAA > 0 && (mUsage & TU_RENDERTARGET))
+		{
+			mFSAALevelSupported = _canUseFSAALevel(usage, D3DRTYPE_CUBETEXTURE, d3dPF, mFSAA);
+		}
 		// check if mip map cube textures are supported
 		mMipmapsHardwareGenerated = false;
 		if (mDevCaps.TextureCaps & D3DPTEXTURECAPS_MIPCUBEMAP)
@@ -708,7 +867,8 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture", "D3D9Texture::_createCubeTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createCubeTex" );
 		}
 
 		// set the base texture we'll use in the render system
@@ -716,7 +876,8 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture", "D3D9Texture::_createCubeTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createCubeTex" );
 		}
 		
 		// set final tex. attributes from tex. description
@@ -726,8 +887,28 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description", "D3D9Texture::_createCubeTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createCubeTex" );
 		}
+
+		if (mFSAA > 0 && mFSAALevelSupported)
+		{
+			// create AA surface
+			HRESULT hr = mpDev->CreateRenderTarget(desc.Width, desc.Height, d3dPF, 
+				(D3DMULTISAMPLE_TYPE)mFSAA, 
+				0, // Only supporting regular sampling, not nonmaskable 
+				FALSE, // not lockable
+				&mFSAASurface, NULL);
+
+			if (FAILED(hr))
+			{
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
+					"Unable to create AA render target: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_createCubeTex");
+			}
+
+		}
+
 		this->_setFinalAttributes(desc.Width, desc.Height, 1, D3D9Mappings::_getPF(desc.Format));
 
 		// Set best filter type
@@ -736,9 +917,11 @@ namespace Ogre
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
 			{
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type", "D3D9Texture::_createCubeTex" );
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_createCubeTex" );
 			}
 		}
+
 	}
 	/****************************************************************************************/
 	void D3D9Texture::_createVolumeTex()
@@ -769,6 +952,13 @@ namespace Ogre
 				mDynamicTextures = false;
 			}
 		}
+		// Check sRGB support
+		if (mHwGamma)
+		{
+			mHwGammaReadSupported = _canUseHardwareGammaCorrection(usage, D3DRTYPE_VOLUMETEXTURE, d3dPF, false);
+			if (mUsage & TU_RENDERTARGET)
+				mHwGammaWriteSupported = _canUseHardwareGammaCorrection(usage, D3DRTYPE_VOLUMETEXTURE, d3dPF, true);
+		}
 		// check if mip map volume textures are supported
 		mMipmapsHardwareGenerated = false;
 		if (mDevCaps.TextureCaps & D3DPTEXTURECAPS_MIPVOLUMEMAP)
@@ -806,7 +996,8 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture", "D3D9Texture::_createVolumeTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Error creating texture: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createVolumeTex" );
 		}
 
 		// set the base texture we'll use in the render system
@@ -814,7 +1005,8 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture", "D3D9Texture::_createVolumeTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get base texture: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createVolumeTex" );
 		}
 		
 		// set final tex. attributes from tex. description
@@ -824,7 +1016,8 @@ namespace Ogre
 		if (FAILED(hr))
 		{
 			this->freeInternalResources();
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description", "D3D9Texture::_createVolumeTex" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Can't get texture description: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_createVolumeTex" );
 		}
 		this->_setFinalAttributes(desc.Width, desc.Height, desc.Depth, D3D9Mappings::_getPF(desc.Format));
 		
@@ -834,7 +1027,8 @@ namespace Ogre
 			hr = mpTex->SetAutoGenFilterType(_getBestFilterMethod());
 			if(FAILED(hr))
 			{
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type", "D3D9Texture::_createCubeTex" );
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Could not set best autogen filter type: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9Texture::_createCubeTex" );
 			}
 		}
 	}
@@ -847,19 +1041,22 @@ namespace Ogre
 		// get device caps
 		hr = mpDev->GetDeviceCaps(&mDevCaps);
 		if (FAILED(hr))
-			OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Can't get device description", "D3D9Texture::_setDevice" );
+			OGRE_EXCEPT( Exception::ERR_INTERNAL_ERROR, "Can't get device description: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_setDevice" );
 
 		// get D3D pointer
 		hr = mpDev->GetDirect3D(&mpD3D);
 		// decrement reference count
 		mpD3D->Release();
 		if (FAILED(hr))
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 pointer", "D3D9Texture::_setDevice" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 pointer: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_setDevice" );
 
 		// get our device creation parameters
 		hr = mpDev->GetCreationParameters(&mDevCreParams);
 		if (FAILED(hr))
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device creation parameters", "D3D9Texture::_setDevice" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device creation parameters: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_setDevice" );
 
 		// get our back buffer pixel format
 		IDirect3DSurface9 *pSrf;
@@ -868,12 +1065,14 @@ namespace Ogre
 		// decrement reference count
 		pSrf->Release();
 		if (FAILED(hr))
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device pixel format", "D3D9Texture::_setDevice" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device pixel format: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_setDevice" );
 
 		hr = pSrf->GetDesc(&srfDesc);
 		if (FAILED(hr))
 		{
-			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device pixel format", "D3D9Texture::_setDevice" );
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "Failed to get D3D9 device pixel format: " + String(DXGetErrorDescription9(hr)), 
+				"D3D9Texture::_setDevice" );
 		}
 
 		mBBPixelFormat = srfDesc.Format;
@@ -914,6 +1113,7 @@ namespace Ogre
 		mSrcDepth = depth;
         mSrcFormat = format;
 		// say to the world what we are doing
+        if (!TextureManager::getSingleton().getVerbose()) return;
 		switch (this->getTextureType())
 		{
 		case TEX_TYPE_1D:
@@ -1001,6 +1201,60 @@ namespace Ogre
 			return true;
 		else
 			return false;
+	}
+	/****************************************************************************************/
+	bool D3D9Texture::_canUseHardwareGammaCorrection(DWORD srcUsage, 
+		D3DRESOURCETYPE srcType, D3DFORMAT srcFormat, bool forwriting)
+	{
+		// those MUST be initialized !!!
+		assert(mpDev);
+		assert(mpD3D);
+
+
+		// Always check 'read' capability here
+		// We will check 'write' capability only in the context of a render target
+		if (forwriting)
+			srcUsage |= D3DUSAGE_QUERY_SRGBWRITE;
+		else
+			srcUsage |= D3DUSAGE_QUERY_SRGBREAD;
+
+		// Check for sRGB support
+		HRESULT hr;
+		// check for auto gen. mip maps support
+		hr = mpD3D->CheckDeviceFormat(
+			mDevCreParams.AdapterOrdinal, 
+			mDevCreParams.DeviceType, 
+			mBBPixelFormat, 
+			srcUsage,
+			srcType,
+			srcFormat);
+		if (hr == D3D_OK)
+			return true;
+		else
+			return false;
+
+	}
+	/****************************************************************************************/
+	bool D3D9Texture::_canUseFSAALevel(DWORD srcUsage, D3DRESOURCETYPE srcType, D3DFORMAT srcFormat, uint fsaa)
+	{
+		// those MUST be initialized !!!
+		assert(mpDev);
+		assert(mpD3D);
+
+
+		HRESULT hr = mpD3D->CheckDeviceMultiSampleType(
+			mDevCreParams.AdapterOrdinal, 
+			mDevCreParams.DeviceType, 
+			srcFormat, 
+			false,
+			(D3DMULTISAMPLE_TYPE)fsaa,
+			0);
+		if (hr == D3D_OK)
+			return true;
+		else
+			return false;
+
+
 	}
 	/****************************************************************************************/
 	bool D3D9Texture::_canAutoGenMipmaps(DWORD srcUsage, D3DRESOURCETYPE srcType, D3DFORMAT srcFormat)
@@ -1105,7 +1359,8 @@ namespace Ogre
 				// this is safe because the texture keeps a reference as well
 				surface->Release();
 
-				GETLEVEL(0, mip)->bind(mpDev, surface, updateOldList);
+				GETLEVEL(0, mip)->bind(mpDev, surface, updateOldList, mHwGammaWriteSupported, 
+					mFSAALevelSupported? mFSAA : 0, mFSAASurface, mName);
 			}
 			break;
 		case TEX_TYPE_CUBE_MAP:
@@ -1122,7 +1377,8 @@ namespace Ogre
 					// this is safe because the texture keeps a reference as well
 					surface->Release();
 					
-					GETLEVEL(face, mip)->bind(mpDev, surface, updateOldList);
+					GETLEVEL(face, mip)->bind(mpDev, surface, updateOldList, mHwGammaWriteSupported, 
+						mFSAALevelSupported? mFSAA : 0, mFSAASurface, mName);
 				}
 			}
 			break;
@@ -1138,7 +1394,7 @@ namespace Ogre
 				// this is safe because the texture keeps a reference as well
 				volume->Release();
 						
-				GETLEVEL(0, mip)->bind(mpDev, volume, updateOldList);
+				GETLEVEL(0, mip)->bind(mpDev, volume, updateOldList, mHwGammaWriteSupported, mName);
 			}
 			break;
 		};
@@ -1166,8 +1422,11 @@ namespace Ogre
 		assert(idx < mSurfaceList.size());
 		return mSurfaceList[idx];
 	}
-	
-	
+	/****************************************************************************************/
+	bool D3D9Texture::useDefaultPool()
+	{
+		return (mUsage & TU_RENDERTARGET) || (mUsage & TU_DYNAMIC);
+	}
 	/****************************************************************************************/
 	bool D3D9Texture::releaseIfDefaultPool(void)
 	{
@@ -1235,4 +1494,60 @@ namespace Ogre
 
         RenderTexture::update();
     }
+	//---------------------------------------------------------------------
+	void D3D9RenderTexture::getCustomAttribute( const String& name, void *pData )
+	{
+		if(name == "DDBACKBUFFER")
+		{
+			if (mFSAA > 0)
+			{
+				// rendering to AA surface
+				IDirect3DSurface9 ** pSurf = (IDirect3DSurface9 **)pData;
+				*pSurf = static_cast<D3D9HardwarePixelBuffer*>(mBuffer)->getFSAASurface();
+				return;
+			}
+			else
+			{
+				IDirect3DSurface9 ** pSurf = (IDirect3DSurface9 **)pData;
+				*pSurf = static_cast<D3D9HardwarePixelBuffer*>(mBuffer)->getSurface();
+				return;
+			}
+		}
+		else if(name == "HWND")
+		{
+			HWND *pHwnd = (HWND*)pData;
+			*pHwnd = NULL;
+			return;
+		}
+		else if(name == "BUFFER")
+		{
+			*static_cast<HardwarePixelBuffer**>(pData) = mBuffer;
+			return;
+		}
+	}
+	//---------------------------------------------------------------------
+	void D3D9RenderTexture::swapBuffers(bool waitForVSync /* = true */)
+	{
+		// Only needed if we have to blit from AA surface
+		if (mFSAA > 0)
+		{
+			D3D9RenderSystem* rs = static_cast<D3D9RenderSystem*>(
+				Root::getSingleton().getRenderSystem());
+			if (rs->isDeviceLost())
+				return;
+
+			D3D9HardwarePixelBuffer* buf = static_cast<D3D9HardwarePixelBuffer*>(mBuffer);
+			HRESULT hr = rs->getDevice()->StretchRect(buf->getFSAASurface(), 0, buf->getSurface(), 0, D3DTEXF_NONE);
+			if (FAILED(hr))
+			{
+				OGRE_EXCEPT(Exception::ERR_INTERNAL_ERROR, 
+					"Unable to copy AA buffer to final buffer: " + String(DXGetErrorDescription9(hr)), 
+					"D3D9RenderTexture::swapBuffers");
+			}
+			
+
+		}
+	}
+
+
 }
