@@ -33,6 +33,7 @@ Torus Knot Software Ltd.
 #include "OgreGLSLGpuProgram.h"
 #include "OgreGLSLProgram.h"
 #include "OgreGLSLLinkProgramManager.h"
+#include "OgreException.h"
 
 namespace Ogre {
 
@@ -72,15 +73,51 @@ namespace Ogre {
 		CustomAttribute("binormal", GLGpuProgram::getFixedAttributeIndex(VES_BINORMAL, 0)),
 	};
 
+	GLint getGLGeometryInputPrimitiveType(RenderOperation::OperationType operationType, bool requiresAdjacency)
+	{
+		switch (operationType)
+		{
+		case RenderOperation::OT_POINT_LIST:
+			return GL_POINTS;
+		case RenderOperation::OT_LINE_LIST:
+		case RenderOperation::OT_LINE_STRIP:
+			return requiresAdjacency ? GL_LINES_ADJACENCY_EXT : GL_LINES;
+		default:
+		case RenderOperation::OT_TRIANGLE_LIST:
+		case RenderOperation::OT_TRIANGLE_STRIP:
+		case RenderOperation::OT_TRIANGLE_FAN:
+			return requiresAdjacency ? GL_TRIANGLES_ADJACENCY_EXT : GL_TRIANGLES;
+		}
+	}
 	//-----------------------------------------------------------------------
-	GLSLLinkProgram::GLSLLinkProgram(GLSLGpuProgram* vertexProgram, GLSLGpuProgram* fragmentProgram)
+	GLint getGLGeometryOutputPrimitiveType(RenderOperation::OperationType operationType)
+	{
+		switch (operationType)
+		{
+		case RenderOperation::OT_POINT_LIST:
+			return GL_POINTS;
+		case RenderOperation::OT_LINE_STRIP:
+			return GL_LINE_STRIP;
+		case RenderOperation::OT_TRIANGLE_STRIP:
+			return GL_TRIANGLE_STRIP;
+		default:
+			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR,
+				"Geometry shader output operation type can only be point list,"
+				"line strip or triangle strip",
+				"GLSLLinkProgram::activate");
+		}
+	}
+	//-----------------------------------------------------------------------
+	GLSLLinkProgram::GLSLLinkProgram(GLSLGpuProgram* vertexProgram, GLSLGpuProgram* geometryProgram, GLSLGpuProgram* fragmentProgram)
         : mVertexProgram(vertexProgram)
+		, mGeometryProgram(geometryProgram)
 		, mFragmentProgram(fragmentProgram)
 		, mUniformRefsBuilt(false)
         , mLinked(false)
 
 	{
-			checkForGLSLError( "GLSLLinkProgram::GLSLLinkProgram", "Error prior to Creating GLSL Program Object", 0 );
+			//checkForGLSLError( "GLSLLinkProgram::GLSLLinkProgram", "Error prior to Creating GLSL Program Object", 0);
+		    glGetError(); //Clean up the error. Otherwise will flood log.
 		    mGLHandle = glCreateProgramObjectARB();
 			checkForGLSLError( "GLSLLinkProgram::GLSLLinkProgram", "Error Creating GLSL Program Object", 0 );
 
@@ -92,7 +129,11 @@ namespace Ogre {
 				mVertexProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
 				setSkeletalAnimationIncluded(mVertexProgram->isSkeletalAnimationIncluded());
 			}
-
+			if (mGeometryProgram)
+			{
+				mGeometryProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
+				//Don't set adjacency flag. We handle it internally and expose "false"
+			}
 			if (mFragmentProgram)
 			{
 				mFragmentProgram->getGLSLProgram()->attachToProgramObject(mGLHandle);
@@ -113,21 +154,57 @@ namespace Ogre {
 		if (!mLinked)
 		{
 			// Some drivers (e.g. OS X on nvidia) incorrectly determine the attribute binding automatically
-			// and end up aliasing existing built-ins. So avoid! Bind all possible attribs
-			// the linker will resolve which ones were actually used
-
+			// and end up aliasing existing built-ins. So avoid! 
+			// Bind all used attribs - not all possible ones otherwise we'll get 
+			// lots of warnings in the log, and also may end up aliasing names used
+			// as varyings by accident
+			// Because we can't ask GL whether an attribute is used in the shader
+			// until it is linked (chicken and egg!) we have to parse the source
+			
 			size_t numAttribs = sizeof(msCustomAttributes)/sizeof(CustomAttribute);
+			const String& vpSource = mVertexProgram->getGLSLProgram()->getSource();
 			for (size_t i = 0; i < numAttribs; ++i)
 			{
 				const CustomAttribute& a = msCustomAttributes[i];
-				glBindAttribLocationARB(mGLHandle, a.attrib, a.name.c_str());
+				
+				// we're looking for 'attribute vec<n> <semantic_name>'
+				// be slightly flexible about formatting
+				String::size_type pos = vpSource.find(a.name);
+				if (pos != String::npos)
+				{
+					String::size_type startpos = vpSource.find("attribute", pos-20);
+					if (startpos != String::npos && startpos < pos)
+						glBindAttribLocationARB(mGLHandle, a.attrib, a.name.c_str());
+				}
+			}
+
+			if (mGeometryProgram)
+			{
+				RenderOperation::OperationType inputOperationType = mGeometryProgram->getGLSLProgram()->getInputOperationType();
+				glProgramParameteriEXT(mGLHandle,GL_GEOMETRY_INPUT_TYPE_EXT,
+					getGLGeometryInputPrimitiveType(inputOperationType, mGeometryProgram->isAdjacencyInfoRequired()));
+				
+				RenderOperation::OperationType outputOperationType = mGeometryProgram->getGLSLProgram()->getOutputOperationType();
+				switch (outputOperationType)
+				{
+				case RenderOperation::OT_POINT_LIST:
+				case RenderOperation::OT_LINE_STRIP:
+				case RenderOperation::OT_TRIANGLE_STRIP:
+					break;
+				
+				}
+				glProgramParameteriEXT(mGLHandle,GL_GEOMETRY_OUTPUT_TYPE_EXT,
+					getGLGeometryOutputPrimitiveType(outputOperationType));
+
+				glProgramParameteriEXT(mGLHandle,GL_GEOMETRY_VERTICES_OUT_EXT,
+					mGeometryProgram->getGLSLProgram()->getMaxOutputVertices());
 			}
 
 			glLinkProgramARB( mGLHandle );
 			glGetObjectParameterivARB( mGLHandle, GL_OBJECT_LINK_STATUS_ARB, &mLinked );
 			// force logging and raise exception if not linked
 			checkForGLSLError( "GLSLLinkProgram::Activate",
-				"Error linking GLSL Program Object", mGLHandle, !mLinked, !mLinked );
+				"Error linking GLSL Program Object : ", mGLHandle, !mLinked, !mLinked );
 			if(mLinked)
 			{
 				logObjectInfo( String("GLSL link result : "), mGLHandle );
@@ -139,7 +216,13 @@ namespace Ogre {
 
 		if (mLinked)
 		{
+			checkForGLSLError( "GLSLLinkProgram::Activate",
+				"Error prior to using GLSL Program Object : ", mGLHandle, false, false);
+
 		    glUseProgramObjectARB( mGLHandle );
+
+			checkForGLSLError( "GLSLLinkProgram::Activate",
+				"Error using GLSL Program Object : ", mGLHandle, false, false);
 		}
 	}
 
@@ -176,18 +259,22 @@ namespace Ogre {
 		{
 			const GpuConstantDefinitionMap* vertParams = 0;
 			const GpuConstantDefinitionMap* fragParams = 0;
-
+			const GpuConstantDefinitionMap* geomParams = 0;
 			if (mVertexProgram)
 			{
 				vertParams = &(mVertexProgram->getGLSLProgram()->getConstantDefinitions().map);
 			}
+			if (mGeometryProgram)
+			{
+				geomParams = &(mGeometryProgram->getGLSLProgram()->getConstantDefinitions().map);
+			}
 			if (mFragmentProgram)
 			{
 				fragParams = &(mFragmentProgram->getGLSLProgram()->getConstantDefinitions().map);
 			}
 
 			GLSLLinkProgramManager::getSingleton().extractUniforms(
-				mGLHandle, vertParams, fragParams, mGLUniformReferences);
+				mGLHandle, vertParams, geomParams, fragParams, mGLUniformReferences);
 
 			mUniformRefsBuilt = true;
 		}
@@ -286,19 +373,19 @@ namespace Ogre {
 					break;
 				case GCT_INT1:
 					glUniform1ivARB(currentUniform->mLocation, glArraySize, 
-						params->getIntPointer(def->physicalIndex));
+						(GLint*)params->getIntPointer(def->physicalIndex));
 					break;
 				case GCT_INT2:
 					glUniform2ivARB(currentUniform->mLocation, glArraySize, 
-						params->getIntPointer(def->physicalIndex));
+						(GLint*)params->getIntPointer(def->physicalIndex));
 					break;
 				case GCT_INT3:
 					glUniform3ivARB(currentUniform->mLocation, glArraySize, 
-						params->getIntPointer(def->physicalIndex));
+						(GLint*)params->getIntPointer(def->physicalIndex));
 					break;
 				case GCT_INT4:
 					glUniform4ivARB(currentUniform->mLocation, glArraySize, 
-						params->getIntPointer(def->physicalIndex));
+						(GLint*)params->getIntPointer(def->physicalIndex));
 					break;
 				case GCT_SAMPLER1D:
 				case GCT_SAMPLER1DSHADOW:
@@ -308,7 +395,7 @@ namespace Ogre {
 				case GCT_SAMPLERCUBE:
 					// samplers handled like 1-element ints
 					glUniform1ivARB(currentUniform->mLocation, 1, 
-						params->getIntPointer(def->physicalIndex));
+						(GLint*)params->getIntPointer(def->physicalIndex));
 					break;
 
 				} // end switch
