@@ -39,12 +39,17 @@ Torus Knot Software Ltd.
 #include "OgreGLSLGpuProgram.h"
 #include "OgreGLSLExtSupport.h"
 #include "OgreGLSLLinkProgramManager.h"
+#include "OgreGLSLPreprocessor.h"
 
 namespace Ogre {
 
     //-----------------------------------------------------------------------
 	GLSLProgram::CmdPreprocessorDefines GLSLProgram::msCmdPreprocessorDefines;
     GLSLProgram::CmdAttach GLSLProgram::msCmdAttach;
+	GLSLProgram::CmdInputOperationType GLSLProgram::msInputOperationTypeCmd;
+	GLSLProgram::CmdOutputOperationType GLSLProgram::msOutputOperationTypeCmd;
+	GLSLProgram::CmdMaxOutputVertices GLSLProgram::msMaxOutputVerticesCmd;
+
     //-----------------------------------------------------------------------
     //---------------------------------------------------------------------------
     GLSLProgram::~GLSLProgram()
@@ -61,27 +66,38 @@ namespace Ogre {
         }
     }
     //-----------------------------------------------------------------------
-    void GLSLProgram::loadFromSource(void)
-    {
-        // only create a shader object if glsl is supported
-        if (isSupported())
-        {
-            checkForGLSLError( "GLSLProgram::GLSLProgram", "GL Errors before creating shader object", 0 );
-            // create shader object
-            mGLHandle = glCreateShaderObjectARB(
-                (mType == GPT_VERTEX_PROGRAM) ? GL_VERTEX_SHADER_ARB : GL_FRAGMENT_SHADER_ARB );
+	void GLSLProgram::loadFromSource(void)
+	{
+		// only create a shader object if glsl is supported
+		if (isSupported())
+		{
+			checkForGLSLError( "GLSLProgram::GLSLProgram", "GL Errors before creating shader object", 0 );
+			// create shader object
 
-            checkForGLSLError( "GLSLProgram::GLSLProgram", "Error creating GLSL shader Object", 0 );
-        }
+			GLenum shaderType;
+			switch (mType)
+			{
+			case GPT_VERTEX_PROGRAM:
+				shaderType = GL_VERTEX_SHADER_ARB;
+				break;
+			case GPT_FRAGMENT_PROGRAM:
+				shaderType = GL_FRAGMENT_SHADER_ARB;
+				break;
+			case GPT_GEOMETRY_PROGRAM:
+				shaderType = GL_GEOMETRY_SHADER_EXT;
+				break;
+			}
+			mGLHandle = glCreateShaderObjectARB(shaderType);
 
-		// build preprocessor defines
-		// GLSL has no explicit interface for defining preprocessor flags outside
-		// of the source code, so we'll prepend some extra source code
-		String preprocessorSource;
-		if (!mPreprocessorDefines.empty())
+			checkForGLSLError( "GLSLProgram::GLSLProgram", "Error creating GLSL shader Object", 0 );
+		}
+
+		// Preprocess the GLSL shader in order to get a clean source
+		CPreprocessor cpp;
+
+		// Pass all user-defined macros to preprocessor
+		if (!mPreprocessorDefines.empty ())
 		{
-			StringUtil::StrStreamType tempStr;
-			// Split preprocessor defines and build up macro array
 			String::size_type pos = 0;
 			while (pos != String::npos)
 			{
@@ -89,57 +105,71 @@ namespace Ogre {
 				String::size_type endPos = mPreprocessorDefines.find_first_of(";,=", pos);
 				if (endPos != String::npos)
 				{
-					// ok, we have a definition
-					tempStr << "#define ";
-					// name
-					tempStr << mPreprocessorDefines.substr(pos, endPos - pos) << " ";
+					String::size_type macro_name_start = pos;
+					size_t macro_name_len = endPos - pos;
 					pos = endPos;
-					
+
 					// Check definition part
 					if (mPreprocessorDefines[pos] == '=')
 					{
 						// set up a definition, skip delim
 						++pos;
+						String::size_type macro_val_start = pos;
+						size_t macro_val_len;
+
 						endPos = mPreprocessorDefines.find_first_of(";,", pos);
 						if (endPos == String::npos)
 						{
-							tempStr << mPreprocessorDefines.substr(pos, String::npos);
+							macro_val_len = mPreprocessorDefines.size () - pos;
 							pos = endPos;
 						}
 						else
 						{
-							tempStr << mPreprocessorDefines.substr(pos, endPos - pos);
+							macro_val_len = endPos - pos;
 							pos = endPos+1;
 						}
+						cpp.Define (
+							mPreprocessorDefines.c_str () + macro_name_start, macro_name_len,
+							mPreprocessorDefines.c_str () + macro_val_start, macro_val_len);
 					}
 					else
 					{
 						// No definition part, define as "1"
 						++pos;
-						tempStr << "1";
+						cpp.Define (
+							mPreprocessorDefines.c_str () + macro_name_start, macro_name_len, 1);
 					}
-
-					tempStr << std::endl;
 				}
 				else
-				{
 					pos = endPos;
-				}
 			}
-
-			preprocessorSource = tempStr.str();
 		}
 
+		size_t out_size = 0;
+		const char *src = mSource.c_str ();
+		size_t src_len = mSource.size ();
+		char *out = cpp.Parse (src, src_len, out_size);
+		if (!out || !out_size)
+			// Failed to preprocess, break out
+			OGRE_EXCEPT (Exception::ERR_RENDERINGAPI_ERROR,
+						 "Failed to preprocess shader " + mName,
+						 __FUNCTION__);
+
+		mSource = String (out, out_size);
+		if (out < src || out > src + src_len)
+			free (out);
+
 		// Add preprocessor extras and main source
-		const char* sources[2];
-		sources[0] = preprocessorSource.c_str();
-		sources[1] = mSource.c_str();
-		glShaderSourceARB(mGLHandle, 2, sources, NULL);
-		// check for load errors
-		checkForGLSLError( "GLSLProgram::loadFromSource", "Cannot load GLSL high-level shader source : " + mName, 0 );
+		if (!mSource.empty())
+		{
+			const char *source = mSource.c_str();
+			glShaderSourceARB(mGLHandle, 1, &source, NULL);
+			// check for load errors
+			checkForGLSLError( "GLSLProgram::loadFromSource", "Cannot load GLSL high-level shader source : " + mName, 0 );
+		}
 
 		compile();
-    }
+	}
     
     //---------------------------------------------------------------------------
 	bool GLSLProgram::compile(const bool checkErrors)
@@ -182,25 +212,23 @@ namespace Ogre {
 		unloadHighLevel();
 	}
 	//-----------------------------------------------------------------------
-    void GLSLProgram::unloadHighLevelImpl(void)
-    {
+	void GLSLProgram::unloadHighLevelImpl(void)
+	{
 		if (isSupported())
 		{
 			glDeleteObjectARB(mGLHandle);
 		}
-
-
-    }
+	}
 
 	//-----------------------------------------------------------------------
-    void GLSLProgram::populateParameterNames(GpuProgramParametersSharedPtr params)
-    {
+	void GLSLProgram::populateParameterNames(GpuProgramParametersSharedPtr params)
+	{
 		params->_setNamedConstants(&getConstantDefinitions());
 		// Don't set logical / physical maps here, as we can't access parameters by logical index in GLHL.
-    }
+	}
 	//-----------------------------------------------------------------------
-    void GLSLProgram::buildConstantDefinitions() const
-    {
+	void GLSLProgram::buildConstantDefinitions() const
+	{
 		// We need an accurate list of all the uniforms in the shader, but we
 		// can't get at them until we link all the shaders into a program object.
 
@@ -208,7 +236,8 @@ namespace Ogre {
 		// Therefore instead, parse the source code manually and extract the uniforms
 		mConstantDefs.floatBufferSize = 0;
 		mConstantDefs.intBufferSize = 0;
-		GLSLLinkProgramManager::getSingleton().extractConstantDefs(mSource, mConstantDefs, mName);
+		GLSLLinkProgramManager::getSingleton().extractConstantDefs(
+			mSource, mConstantDefs, mName);
 
 		// Also parse any attached sources
 		for (GLSLProgramContainer::const_iterator i = mAttachedGLSLPrograms.begin();
@@ -220,16 +249,15 @@ namespace Ogre {
 				childShader->getSource(), mConstantDefs, childShader->getName());
 
 		}
-
-
-
-    }
+	}
 
 	//-----------------------------------------------------------------------
     GLSLProgram::GLSLProgram(ResourceManager* creator, 
         const String& name, ResourceHandle handle,
         const String& group, bool isManual, ManualResourceLoader* loader)
-        : HighLevelGpuProgram(creator, name, handle, group, isManual, loader)
+        : HighLevelGpuProgram(creator, name, handle, group, isManual, loader),
+            mInputOperationType(RenderOperation::OT_TRIANGLE_LIST),
+            mOutputOperationType(RenderOperation::OT_TRIANGLE_LIST), mMaxOutputVertices(3)
     {
 		// add parameter command "attach" to the material serializer dictionary
         if (createParamDictionary("GLSLProgram"))
@@ -243,6 +271,22 @@ namespace Ogre {
             dict->addParameter(ParameterDef("attach", 
                 "name of another GLSL program needed by this program",
                 PT_STRING),&msCmdAttach);
+			dict->addParameter(
+				ParameterDef("input_operation_type",
+				"The input operation type for this geometry program. \
+				Can be 'point_list', 'line_list', 'line_strip', 'triangle_list', \
+				'triangle_strip' or 'triangle_fan'", PT_STRING),
+				&msInputOperationTypeCmd);
+			dict->addParameter(
+				ParameterDef("output_operation_type",
+				"The input operation type for this geometry program. \
+				Can be 'point_list', 'line_strip' or 'triangle_strip'",
+				 PT_STRING),
+				 &msOutputOperationTypeCmd);
+			dict->addParameter(
+				ParameterDef("max_output_vertices", 
+				"The maximum number of vertices a single run of this geometry program can output",
+				PT_INT),&msMaxOutputVerticesCmd);
         }
         // Manually assign language now since we use it immediately
         mSyntaxCode = "glsl";
@@ -359,6 +403,94 @@ namespace Ogre {
 
         return language;
     }
+	//-----------------------------------------------------------------------
+	RenderOperation::OperationType parseOperationType(const String& val)
+	{
+		if (val == "point_list")
+		{
+			return RenderOperation::OT_POINT_LIST;
+		}
+		else if (val == "line_list")
+		{
+			return RenderOperation::OT_LINE_LIST;
+		}
+		else if (val == "line_strip")
+		{
+			return RenderOperation::OT_LINE_STRIP;
+		}
+		else if (val == "triangle_strip")
+		{
+			return RenderOperation::OT_TRIANGLE_STRIP;
+		}
+		else if (val == "triangle_fan")
+		{
+			return RenderOperation::OT_TRIANGLE_FAN;
+		}
+		else 
+		{
+			//Triangle list is the default fallback. Keep it this way?
+			return RenderOperation::OT_TRIANGLE_LIST;
+		}
+	}
+	//-----------------------------------------------------------------------
+	String operationTypeToString(RenderOperation::OperationType val)
+	{
+		switch (val)
+		{
+		case RenderOperation::OT_POINT_LIST:
+			return "point_list";
+			break;
+		case RenderOperation::OT_LINE_LIST:
+			return "line_list";
+			break;
+		case RenderOperation::OT_LINE_STRIP:
+			return "line_strip";
+			break;
+		case RenderOperation::OT_TRIANGLE_STRIP:
+			return "triangle_strip";
+			break;
+		case RenderOperation::OT_TRIANGLE_FAN:
+			return "triangle_fan";
+			break;
+		case RenderOperation::OT_TRIANGLE_LIST:
+		default:
+			return "triangle_list";
+			break;
+		}
+	}
+	//-----------------------------------------------------------------------
+    String GLSLProgram::CmdInputOperationType::doGet(const void* target) const
+    {
+        const GLSLProgram* t = static_cast<const GLSLProgram*>(target);
+		return operationTypeToString(t->getInputOperationType());
+    }
+    void GLSLProgram::CmdInputOperationType::doSet(void* target, const String& val)
+    {
+        GLSLProgram* t = static_cast<GLSLProgram*>(target);
+		t->setInputOperationType(parseOperationType(val));
+    }
+	//-----------------------------------------------------------------------
+	String GLSLProgram::CmdOutputOperationType::doGet(const void* target) const
+    {
+        const GLSLProgram* t = static_cast<const GLSLProgram*>(target);
+		return operationTypeToString(t->getOutputOperationType());
+    }
+    void GLSLProgram::CmdOutputOperationType::doSet(void* target, const String& val)
+    {
+        GLSLProgram* t = static_cast<GLSLProgram*>(target);
+		t->setOutputOperationType(parseOperationType(val));
+    }
+	//-----------------------------------------------------------------------
+	String GLSLProgram::CmdMaxOutputVertices::doGet(const void* target) const
+	{
+		const GLSLProgram* t = static_cast<const GLSLProgram*>(target);
+		return StringConverter::toString(t->getMaxOutputVertices());
+	}
+	void GLSLProgram::CmdMaxOutputVertices::doSet(void* target, const String& val)
+	{
+		GLSLProgram* t = static_cast<GLSLProgram*>(target);
+		t->setMaxOutputVertices(StringConverter::parseInt(val));
+	}
 
   
 }
