@@ -35,6 +35,7 @@ Torus Knot Software Ltd.
 #include "OgreException.h"
 #include "OgreLogManager.h"
 #include "OgreStringConverter.h"
+#include "OgreGLPixelFormat.h"
 
 #include <OpenGL/gl.h>
 #define GL_EXT_texture_env_combine 1
@@ -55,48 +56,54 @@ OSXWindow::~OSXWindow()
 }
 
 //-------------------------------------------------------------------------------------------------//
-void OSXWindow::writeContentsToFile(const String& filename)
+void OSXWindow::copyContentsToMemory(const PixelBox &dst, FrameBuffer buffer)
 {
-	ImageCodec::ImageData* imgData = new ImageCodec::ImageData;
-	imgData->width = mWidth;
-	imgData->height = mHeight;
-	imgData->format = PF_BYTE_RGB;
+	if ((dst.left < 0) || (dst.right > mWidth) ||
+		(dst.top < 0) || (dst.bottom > mHeight) ||
+		(dst.front != 0) || (dst.back != 1))
+	{
+		OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+					"Invalid box.",
+					"OSXWindow::copyContentsToMemory" );
+	}
 
-	// Allocate buffer
-	uchar* pBuffer = new uchar[mWidth * mHeight * 3];
+	if (buffer == FB_AUTO)
+	{
+		buffer = mIsFullScreen? FB_FRONT : FB_BACK;
+	}
 
-	// Read pixels
-	// I love GL: it does all the locking & colour conversion for us
-	glReadPixels(0,0, mWidth-1, mHeight-1, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
+	GLenum format = Ogre::GLPixelUtil::getGLOriginFormat(dst.format);
+	GLenum type = Ogre::GLPixelUtil::getGLOriginDataType(dst.format);
 
-	// Wrap buffer in a memory stream
-	DataStreamPtr stream(new MemoryDataStream(pBuffer, mWidth * mHeight * 3, false));
+	if ((format == GL_NONE) || (type == 0))
+	{
+		OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+					"Unsupported format.",
+					"OSXWindow::copyContentsToMemory" );
+	}
 
-	// Need to flip the read data over in Y though
-	Image img;
-	img.loadRawData(stream, mWidth, mHeight, PF_BYTE_RGB );
-	img.flipAroundX();
+	glReadBuffer((buffer == FB_FRONT)? GL_FRONT : GL_BACK);
+	glReadPixels((GLint)dst.left, (GLint)dst.top,
+				 (GLsizei)dst.getWidth(), (GLsizei)dst.getHeight(),
+				 format, type, dst.data);
 
-	MemoryDataStreamPtr streamFlipped(new MemoryDataStream(img.getData(), stream->size(), false));
+	//vertical flip
+	{
+		size_t rowSpan = dst.getWidth() * PixelUtil::getNumElemBytes(dst.format);
+		size_t height = dst.getHeight();
+		uchar *tmpData = new uchar[rowSpan * height];
+		uchar *srcRow = (uchar *)dst.data, *tmpRow = tmpData + (height - 1) * rowSpan;
 
-	// Get codec
-	size_t pos = filename.find_last_of(".");
-	String extension;
-	if( pos == String::npos )
-		OGRE_EXCEPT( Exception::ERR_INVALIDPARAMS, "Unable to determine image type for '" 
-			+ filename + "' - invalid extension.", "OSXCarbonWindow::writeContentsToFile" );
+		while (tmpRow >= tmpData)
+		{
+			memcpy(tmpRow, srcRow, rowSpan);
+			srcRow += rowSpan;
+			tmpRow -= rowSpan;
+		}
+		memcpy(dst.data, tmpData, rowSpan * height);
 
-	while( pos != filename.length() - 1 )
-		extension += filename[++pos];
-
-	// Get the codec
-	Codec * pCodec = Codec::getCodec(extension);
-
-	// Write out
-	Codec::CodecDataPtr codecDataPtr(imgData);
-	pCodec->codeToFile(streamFlipped, filename, codecDataPtr);
-
-	delete [] pBuffer;
+		delete [] tmpData;
+	}
 }
 
 //-------------------------------------------------------------------------------------------------//
