@@ -39,9 +39,9 @@ namespace Ogre {
 
 //-----------------------------------------------------------------------------    
     GLFBORenderTexture::GLFBORenderTexture(GLFBOManager *manager, const String &name,
-        const GLSurfaceDesc &target):
-        GLRenderTexture(name, target),
-        mFB(manager)
+        const GLSurfaceDesc &target, bool writeGamma, uint fsaa):
+        GLRenderTexture(name, target, writeGamma, fsaa),
+        mFB(manager, fsaa)
     {
         // Bind target to surface 0 and initialise
         mFB.bindSurface(0, target);
@@ -57,6 +57,11 @@ namespace Ogre {
             *static_cast<GLFrameBufferObject **>(pData) = &mFB;
         }
     }
+
+	void GLFBORenderTexture::swapBuffers(bool waitForVSync)
+	{
+		mFB.swapBuffers();
+	}
    
 /// Size of probe texture
 #define PROBE_SIZE 16
@@ -406,9 +411,10 @@ static const size_t depthBits[] =
         *stencilFormat = stencilFormats[props.modes[bestmode].stencil];
     }
 
-    GLFBORenderTexture *GLFBOManager::createRenderTexture(const String &name, const GLSurfaceDesc &target)
+    GLFBORenderTexture *GLFBOManager::createRenderTexture(const String &name, 
+		const GLSurfaceDesc &target, bool writeGamma, uint fsaa)
     {
-        GLFBORenderTexture *retval = new GLFBORenderTexture(this, name, target);
+        GLFBORenderTexture *retval = new GLFBORenderTexture(this, name, target, writeGamma, fsaa);
         return retval;
     }
 	MultiRenderTarget *GLFBOManager::createMultiRenderTarget(const String & name)
@@ -416,15 +422,6 @@ static const size_t depthBits[] =
 		return new GLFBOMultiRenderTarget(this, name);
 	}
 
-    GLFrameBufferObject *GLFBOManager::createFrameBufferObject()
-    {
-        return new GLFrameBufferObject(this);
-    }
-
-    void GLFBOManager::destroyFrameBufferObject(GLFrameBufferObject * x)
-    {
-        delete x;
-    }
     void GLFBOManager::bind(RenderTarget *target)
     {
         /// Check if the render target is in the rendertarget->FBO map
@@ -437,28 +434,30 @@ static const size_t depthBits[] =
             glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
     }
     
-    GLSurfaceDesc GLFBOManager::requestRenderBuffer(GLenum format, size_t width, size_t height)
+    GLSurfaceDesc GLFBOManager::requestRenderBuffer(GLenum format, size_t width, size_t height, uint fsaa)
     {
         GLSurfaceDesc retval;
         retval.buffer = 0; // Return 0 buffer if GL_NONE is requested
         if(format != GL_NONE)
         {
-            RBFormat key(format, width, height);
+            RBFormat key(format, width, height, fsaa);
             RenderBufferMap::iterator it = mRenderBufferMap.find(key);
             if(it != mRenderBufferMap.end())
             {
                 retval.buffer = it->second.buffer;
                 retval.zoffset = 0;
+				retval.numSamples = fsaa;
                 // Increase refcount
                 ++it->second.refcount;
             }
             else
             {
                 // New one
-                GLRenderBuffer *rb = new GLRenderBuffer(format, width, height);
+                GLRenderBuffer *rb = new GLRenderBuffer(format, width, height, fsaa);
                 mRenderBufferMap[key] = RBRef(rb);
                 retval.buffer = rb;
                 retval.zoffset = 0;
+				retval.numSamples = fsaa;
             }
         }
         //std::cerr << "Requested renderbuffer with format " << std::hex << format << std::dec << " of " << width << "x" << height << " :" << retval.buffer << std::endl;
@@ -469,7 +468,7 @@ static const size_t depthBits[] =
     {
         if(surface.buffer == 0)
             return;
-        RBFormat key(surface.buffer->getGLFormat(), surface.buffer->getWidth(), surface.buffer->getHeight());
+        RBFormat key(surface.buffer->getGLFormat(), surface.buffer->getWidth(), surface.buffer->getHeight(), surface.numSamples);
         RenderBufferMap::iterator it = mRenderBufferMap.find(key);
         assert(it != mRenderBufferMap.end());
         if (it != mRenderBufferMap.end())   // Just in case
@@ -484,7 +483,7 @@ static const size_t depthBits[] =
     {
         if(surface.buffer == 0)
             return;
-        RBFormat key(surface.buffer->getGLFormat(), surface.buffer->getWidth(), surface.buffer->getHeight());
+        RBFormat key(surface.buffer->getGLFormat(), surface.buffer->getWidth(), surface.buffer->getHeight(), surface.numSamples);
         RenderBufferMap::iterator it = mRenderBufferMap.find(key);
         if(it != mRenderBufferMap.end())
 		{
