@@ -92,7 +92,7 @@ void GLHardwarePixelBuffer::unlockImpl(void)
 	if (mCurrentLockOptions != HardwareBuffer::HBL_READ_ONLY)
 	{
 		// From buffer to card, only upload if was locked for writing
-		upload(mCurrentLock);
+		upload(mCurrentLock, Box(0,0,0,mWidth,mHeight,mDepth));
 	}
 	
 	freeBuffer();
@@ -129,16 +129,9 @@ void GLHardwarePixelBuffer::blitFromMemory(const PixelBox &src, const Image::Box
 	{
 		// No scaling or conversion needed
 		scaled = src;
-		// Set extents for upload
-		scaled.left = dstBox.left;
-		scaled.right = dstBox.right;
-		scaled.top = dstBox.top;
-		scaled.bottom = dstBox.bottom;
-		scaled.front = dstBox.front;
-		scaled.back = dstBox.back;
 	}
 	
-	upload(scaled);
+	upload(scaled, dstBox);
 	freeBuffer();
 }
 //-----------------------------------------------------------------------------  
@@ -181,7 +174,7 @@ void GLHardwarePixelBuffer::blitToMemory(const Image::Box &srcBox, const PixelBo
 	}
 }
 //-----------------------------------------------------------------------------
-void GLHardwarePixelBuffer::upload(const PixelBox &data)
+void GLHardwarePixelBuffer::upload(const PixelBox &data, const Image::Box &dest)
 {
     OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
 		"Upload not possible for this pixelbuffer type",
@@ -200,7 +193,9 @@ void GLHardwarePixelBuffer::bindToFramebuffer(GLenum attachment, size_t zoffset)
         "GLHardwarePixelBuffer::bindToFramebuffer");
 }
 //********* GLTextureBuffer
-GLTextureBuffer::GLTextureBuffer(const String &baseName, GLenum target, GLuint id, GLint face, GLint level, Usage usage, bool crappyCard):
+GLTextureBuffer::GLTextureBuffer(const String &baseName, GLenum target, GLuint id, 
+								 GLint face, GLint level, Usage usage, bool crappyCard, 
+								 bool writeGamma, uint fsaa):
 	GLHardwarePixelBuffer(0, 0, 0, PF_UNKNOWN, usage),
 	mTarget(target), mTextureID(id), mFace(face), mLevel(level), mSoftwareMipmap(crappyCard)
 {
@@ -270,11 +265,11 @@ GLTextureBuffer::GLTextureBuffer(const String &baseName, GLenum target, GLuint i
         for(size_t zoffset=0; zoffset<mDepth; ++zoffset)
         {
             String name;
-			name = "rtt/" + baseName + "/" + StringConverter::toString((size_t)this);
+			name = "rtt/" + StringConverter::toString((size_t)this) + "/" + baseName;
             GLSurfaceDesc target;
             target.buffer = this;
             target.zoffset = zoffset;
-            RenderTexture *trt = GLRTTManager::getSingleton().createRenderTexture(name, target);
+            RenderTexture *trt = GLRTTManager::getSingleton().createRenderTexture(name, target, writeGamma, fsaa);
             mSliceTRT.push_back(trt);
             Root::getSingleton().getRenderSystem()->attachRenderTarget(*mSliceTRT[zoffset]);
         }
@@ -293,7 +288,7 @@ GLTextureBuffer::~GLTextureBuffer()
 	}
 }
 //-----------------------------------------------------------------------------
-void GLTextureBuffer::upload(const PixelBox &data)
+void GLTextureBuffer::upload(const PixelBox &data, const Image::Box &dest)
 {
 	glBindTexture( mTarget, mTextureID );
 	if(PixelUtil::isCompressed(data.format))
@@ -307,46 +302,92 @@ void GLTextureBuffer::upload(const PixelBox &data)
 		// for compressed formats
 		switch(mTarget) {
 			case GL_TEXTURE_1D:
-				glCompressedTexSubImage1DARB(GL_TEXTURE_1D, mLevel, 
-					data.left,
-					data.getWidth(),
-					format, data.getConsecutiveSize(),
-					data.data);
+				// some systems (e.g. old Apple) don't like compressed subimage calls
+				// so prefer non-sub versions
+				if (dest.left == 0)
+				{
+					glCompressedTexImage1DARB(GL_TEXTURE_1D, mLevel,
+						format,
+						dest.getWidth(),
+						0,
+						data.getConsecutiveSize(),
+						data.data);
+				}
+				else
+				{
+					glCompressedTexSubImage1DARB(GL_TEXTURE_1D, mLevel, 
+						dest.left,
+						dest.getWidth(),
+						format, data.getConsecutiveSize(),
+						data.data);
+				}
 				break;
 			case GL_TEXTURE_2D:
 			case GL_TEXTURE_CUBE_MAP:
-				glCompressedTexSubImage2DARB(mFaceTarget, mLevel, 
-					data.left, data.top, 
-					data.getWidth(), data.getHeight(),
-					format, data.getConsecutiveSize(),
-					data.data);
+				// some systems (e.g. old Apple) don't like compressed subimage calls
+				// so prefer non-sub versions
+				if (dest.left == 0 && dest.top == 0)
+				{
+					glCompressedTexImage2DARB(mFaceTarget, mLevel,
+						format,
+						dest.getWidth(),
+						dest.getHeight(),
+						0,
+						data.getConsecutiveSize(),
+						data.data);
+				}
+				else
+				{
+					glCompressedTexSubImage2DARB(mFaceTarget, mLevel, 
+						dest.left, dest.top, 
+						dest.getWidth(), dest.getHeight(),
+						format, data.getConsecutiveSize(),
+						data.data);
+				}
 				break;
 			case GL_TEXTURE_3D:
-				glCompressedTexSubImage3DARB(GL_TEXTURE_3D, mLevel, 
-					data.left, data.top, data.front,
-					data.getWidth(), data.getHeight(), data.getDepth(),
-					format, data.getConsecutiveSize(),
-					data.data);
+				// some systems (e.g. old Apple) don't like compressed subimage calls
+				// so prefer non-sub versions
+				if (dest.left == 0 && dest.top == 0 && dest.front == 0)
+				{
+					glCompressedTexImage3DARB(GL_TEXTURE_3D, mLevel,
+						format,
+						dest.getWidth(),
+						dest.getHeight(),
+						dest.getDepth(),
+						0,
+						data.getConsecutiveSize(),
+						data.data);
+				}
+				else
+				{			
+					glCompressedTexSubImage3DARB(GL_TEXTURE_3D, mLevel, 
+						dest.left, dest.top, dest.front,
+						dest.getWidth(), dest.getHeight(), dest.getDepth(),
+						format, data.getConsecutiveSize(),
+						data.data);
+				}
 				break;
 		}
 		
 	} 
 	else if(mSoftwareMipmap)
 	{
-		GLint internalFormat;
-		glGetTexLevelParameteriv(mTarget, mLevel, GL_TEXTURE_INTERNAL_FORMAT, &internalFormat);
+		GLint components = PixelUtil::getComponentCount(mFormat);
 		if(data.getWidth() != data.rowPitch)
 			glPixelStorei(GL_UNPACK_ROW_LENGTH, data.rowPitch);
 		if(data.getHeight()*data.getWidth() != data.slicePitch)
 			glPixelStorei(GL_UNPACK_IMAGE_HEIGHT, (data.slicePitch/data.getWidth()));
+		if(data.left > 0 || data.top > 0 || data.front > 0)
+			glPixelStorei(GL_UNPACK_SKIP_PIXELS, data.left + data.rowPitch * data.top + data.slicePitch * data.front);
 		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
 		
 		switch(mTarget)
 		{
 		case GL_TEXTURE_1D:
 			gluBuild1DMipmaps(
-				GL_TEXTURE_1D, internalFormat,
-				data.getWidth(),
+				GL_TEXTURE_1D, components,
+				dest.getWidth(),
 				GLPixelUtil::getGLOriginFormat(data.format), GLPixelUtil::getGLOriginDataType(data.format),
 				data.data);
 			break;
@@ -354,7 +395,7 @@ void GLTextureBuffer::upload(const PixelBox &data)
 		case GL_TEXTURE_CUBE_MAP:
 			gluBuild2DMipmaps(
 				mFaceTarget,
-				internalFormat, data.getWidth(), data.getHeight(), 
+				components, dest.getWidth(), dest.getHeight(), 
 				GLPixelUtil::getGLOriginFormat(data.format), GLPixelUtil::getGLOriginDataType(data.format), 
 				data.data);
 			break;		
@@ -368,8 +409,8 @@ void GLTextureBuffer::upload(const PixelBox &data)
 				data.data);
 			*/
 			glTexImage3D(
-				GL_TEXTURE_3D, 0, internalFormat, 
-				data.getWidth(), data.getHeight(), data.getDepth(), 0, 
+				GL_TEXTURE_3D, 0, components, 
+				dest.getWidth(), dest.getHeight(), dest.getDepth(), 0, 
 				GLPixelUtil::getGLOriginFormat(data.format), GLPixelUtil::getGLOriginDataType(data.format),
 				data.data );
 			break;
@@ -381,6 +422,8 @@ void GLTextureBuffer::upload(const PixelBox &data)
 			glPixelStorei(GL_UNPACK_ROW_LENGTH, data.rowPitch);
 		if(data.getHeight()*data.getWidth() != data.slicePitch)
 			glPixelStorei(GL_UNPACK_IMAGE_HEIGHT, (data.slicePitch/data.getWidth()));
+		if(data.left > 0 || data.top > 0 || data.front > 0)
+			glPixelStorei(GL_UNPACK_SKIP_PIXELS, data.left + data.rowPitch * data.top + data.slicePitch * data.front);
 		if((data.getWidth()*PixelUtil::getNumElemBytes(data.format)) & 3) {
 			// Standard alignment of 4 is not right
 			glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
@@ -388,24 +431,24 @@ void GLTextureBuffer::upload(const PixelBox &data)
 		switch(mTarget) {
 			case GL_TEXTURE_1D:
 				glTexSubImage1D(GL_TEXTURE_1D, mLevel, 
-					data.left,
-					data.getWidth(),
+					dest.left,
+					dest.getWidth(),
 					GLPixelUtil::getGLOriginFormat(data.format), GLPixelUtil::getGLOriginDataType(data.format),
 					data.data);
 				break;
 			case GL_TEXTURE_2D:
 			case GL_TEXTURE_CUBE_MAP:
 				glTexSubImage2D(mFaceTarget, mLevel, 
-					data.left, data.top, 
-					data.getWidth(), data.getHeight(),
+					dest.left, dest.top, 
+					dest.getWidth(), dest.getHeight(),
 					GLPixelUtil::getGLOriginFormat(data.format), GLPixelUtil::getGLOriginDataType(data.format),
 					data.data);
 				break;
 			case GL_TEXTURE_3D:
 				glTexSubImage3D(
 					GL_TEXTURE_3D, mLevel, 
-					data.left, data.top, data.front,
-					data.getWidth(), data.getHeight(), data.getDepth(),
+					dest.left, dest.top, dest.front,
+					dest.getWidth(), dest.getHeight(), dest.getDepth(),
 					GLPixelUtil::getGLOriginFormat(data.format), GLPixelUtil::getGLOriginDataType(data.format),
 					data.data);
 				break;
@@ -414,6 +457,7 @@ void GLTextureBuffer::upload(const PixelBox &data)
 	// Restore defaults
 	glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
 	glPixelStorei(GL_UNPACK_IMAGE_HEIGHT, 0);
+	glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
 	glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
 }
 //-----------------------------------------------------------------------------  
@@ -441,6 +485,8 @@ void GLTextureBuffer::download(const PixelBox &data)
 			glPixelStorei(GL_PACK_ROW_LENGTH, data.rowPitch);
 		if(data.getHeight()*data.getWidth() != data.slicePitch)
 			glPixelStorei(GL_PACK_IMAGE_HEIGHT, (data.slicePitch/data.getWidth()));
+		if(data.left > 0 || data.top > 0 || data.front > 0)
+			glPixelStorei(GL_PACK_SKIP_PIXELS, data.left + data.rowPitch * data.top + data.slicePitch * data.front);
 		if((data.getWidth()*PixelUtil::getNumElemBytes(data.format)) & 3) {
 			// Standard alignment of 4 is not right
 			glPixelStorei(GL_PACK_ALIGNMENT, 1);
@@ -452,6 +498,7 @@ void GLTextureBuffer::download(const PixelBox &data)
 		// Restore defaults
 		glPixelStorei(GL_PACK_ROW_LENGTH, 0);
 		glPixelStorei(GL_PACK_IMAGE_HEIGHT, 0);
+		glPixelStorei(GL_PACK_SKIP_PIXELS, 0);
 		glPixelStorei(GL_PACK_ALIGNMENT, 4);
 	}
 }
@@ -781,13 +828,11 @@ void GLTextureBuffer::blitFromMemory(const PixelBox &src_orig, const Image::Box
         glTexImage2D(target, 0, format, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
 
     /// GL texture buffer
-    GLTextureBuffer tex(StringUtil::BLANK, target, id, 0, 0, (Usage)(TU_AUTOMIPMAP|HBU_STATIC_WRITE_ONLY), false);
+    GLTextureBuffer tex(StringUtil::BLANK, target, id, 0, 0, (Usage)(TU_AUTOMIPMAP|HBU_STATIC_WRITE_ONLY), false, false, 0);
     
     /// Upload data to 0,0,0 in temporary texture
-    PixelBox tempTarget(src.getWidth(), src.getHeight(), src.getDepth(), src.format, src.data);
-    tempTarget.rowPitch = src.rowPitch;
-    tempTarget.slicePitch = src.slicePitch;
-    tex.upload(tempTarget);
+	Image::Box tempTarget(0, 0, 0, src.getWidth(), src.getHeight(), src.getDepth());
+    tex.upload(src, tempTarget);
     
     /// Blit
     blitFromTexture(&tex, tempTarget, dstBox);
@@ -805,7 +850,7 @@ RenderTexture *GLTextureBuffer::getRenderTarget(size_t zoffset)
 }
 //********* GLRenderBuffer
 //----------------------------------------------------------------------------- 
-GLRenderBuffer::GLRenderBuffer(GLenum format, size_t width, size_t height):
+GLRenderBuffer::GLRenderBuffer(GLenum format, size_t width, size_t height, GLsizei numSamples):
     GLHardwarePixelBuffer(width, height, 1, GLPixelUtil::getClosestOGREFormat(format),HBU_WRITE_ONLY)
 {
     mGLInternalFormat = format;
@@ -815,8 +860,16 @@ GLRenderBuffer::GLRenderBuffer(GLenum format, size_t width, size_t height):
     glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, mRenderbufferID);
     
     /// Allocate storage for depth buffer
-    glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, format,
-                        width, height);
+	if (numSamples > 0)
+	{
+		glRenderbufferStorageMultisampleEXT(GL_RENDERBUFFER_EXT, 
+			numSamples, format, width, height);
+	}
+	else
+	{
+		glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, format,
+							width, height);
+	}
 }
 //----------------------------------------------------------------------------- 
 GLRenderBuffer::~GLRenderBuffer()
