@@ -138,9 +138,11 @@ namespace Ogre {
 		// problem yet and in fact software generation appears to cause a crash 
 		// in some cases which I've yet to track down
 #if OGRE_PLATFORM != OGRE_PLATFORM_APPLE
-		if (mGLSupport.getGLVendor().find("NVIDIA") != std::string::npos
+		if (Root::getSingleton().getRenderSystem()->getCapabilities()->getVendor() == GPU_NVIDIA
 			&& !PixelUtil::isCompressed(mFormat))
+		{
 			mMipmapsHardwareGenerated = false;
+		}
 #endif
 		if((mUsage & TU_AUTOMIPMAP) &&
 		    mNumRequestedMipmaps && mMipmapsHardwareGenerated)
@@ -150,7 +152,7 @@ namespace Ogre {
 		
 		// Allocate internal buffer so that glTexSubImageXD can be used
 		// Internal format
-		GLenum format = GLPixelUtil::getClosestGLInternalFormat(mFormat);
+		GLenum format = GLPixelUtil::getClosestGLInternalFormat(mFormat, mHwGamma);
 		size_t width = mWidth;
 		size_t height = mHeight;
 		size_t depth = mDepth;
@@ -247,98 +249,110 @@ namespace Ogre {
 		// This already does everything neccessary
         createInternalResources();
     }
+
+    static inline void do_image_io(const String &name, const String &group,
+                                   const String &ext,
+                                   std::vector<Image> &images,
+                                   Resource *r)
+    {
+		size_t imgIdx = images.size();
+        images.push_back(Image());
+
+        DataStreamPtr dstream = 
+            ResourceGroupManager::getSingleton().openResource(
+                name, group, true, r);
+
+        images[imgIdx].load(dstream, ext);
+    }
+
 	
+    void GLTexture::prepareImpl()
+    {
+        if( mUsage & TU_RENDERTARGET ) return;
+
+        String baseName, ext;
+        size_t pos = mName.find_last_of(".");
+        baseName = mName.substr(0, pos);
+        if( pos != String::npos )
+            ext = mName.substr(pos+1);
+
+        LoadedImages loadedImages = LoadedImages(new std::vector<Image>());
+
+        if(mTextureType == TEX_TYPE_1D || mTextureType == TEX_TYPE_2D || 
+            mTextureType == TEX_TYPE_3D)
+        {
+
+            do_image_io(mName, mGroup, ext, *loadedImages, this);
+
+
+            // If this is a cube map, set the texture type flag accordingly.
+            if ((*loadedImages)[0].hasFlag(IF_CUBEMAP))
+                mTextureType = TEX_TYPE_CUBE_MAP;
+            // If this is a volumetric texture set the texture type flag accordingly.
+            if((*loadedImages)[0].getDepth() > 1)
+                mTextureType = TEX_TYPE_3D;
+
+        }
+        else if (mTextureType == TEX_TYPE_CUBE_MAP)
+        {
+            if(getSourceFileType() == "dds")
+            {
+                // XX HACK there should be a better way to specify whether 
+                // all faces are in the same file or not
+                do_image_io(mName, mGroup, ext, *loadedImages, this);
+            }
+            else
+            {
+                std::vector<Image> images(6);
+                ConstImagePtrList imagePtrs;
+                static const String suffixes[6] = {"_rt", "_lf", "_up", "_dn", "_fr", "_bk"};
+
+                for(size_t i = 0; i < 6; i++)
+                {
+                    String fullName = baseName + suffixes[i];
+                    if (!ext.empty())
+                        fullName = fullName + "." + ext;
+                    // find & load resource data intro stream to allow resource
+                    // group changes if required
+                    do_image_io(fullName,mGroup,ext,*loadedImages,this);
+                }
+            }
+        }
+        else
+            OGRE_EXCEPT( Exception::ERR_NOT_IMPLEMENTED, "**** Unknown texture type ****", "GLTexture::prepare" );
+
+        mLoadedImages = loadedImages;
+    }
+	
+    void GLTexture::unprepareImpl()
+    {
+        mLoadedImages.setNull();
+    }
+
     void GLTexture::loadImpl()
     {
         if( mUsage & TU_RENDERTARGET )
         {
             createRenderTexture();
+            return;
         }
-        else
-        {
-			String baseName, ext;
-			size_t pos = mName.find_last_of(".");
-			if( pos == String::npos )
-				OGRE_EXCEPT(
-					Exception::ERR_INVALIDPARAMS, 
-					"Unable to load image file '"+ mName + "' - invalid extension.",
-					"GLTexture::loadImpl" );
 
-			baseName = mName.substr(0, pos);
-			ext = mName.substr(pos+1);
-    
-			if(mTextureType == TEX_TYPE_1D || mTextureType == TEX_TYPE_2D || 
-                mTextureType == TEX_TYPE_3D)
-            {
-                Image img;
-	            // find & load resource data intro stream to allow resource
-				// group changes if required
-				DataStreamPtr dstream = 
-					ResourceGroupManager::getSingleton().openResource(
-						mName, mGroup, true, this);
+        // Now the only copy is on the stack and will be cleaned in case of
+        // exceptions being thrown from _loadImages
+        LoadedImages loadedImages = mLoadedImages;
+        mLoadedImages.setNull();
 
-                img.load(dstream, ext);
-
-				// If this is a cube map, set the texture type flag accordingly.
-                if (img.hasFlag(IF_CUBEMAP))
-					mTextureType = TEX_TYPE_CUBE_MAP;
-				// If this is a volumetric texture set the texture type flag accordingly.
-				if(img.getDepth() > 1)
-					mTextureType = TEX_TYPE_3D;
-
-				// Call internal _loadImages, not loadImage since that's external and 
-				// will determine load status etc again
-				ConstImagePtrList imagePtrs;
-				imagePtrs.push_back(&img);
-				_loadImages( imagePtrs );
-            }
-            else if (mTextureType == TEX_TYPE_CUBE_MAP)
-            {
-				if(StringUtil::endsWith(getName(), ".dds"))
-				{
-					// XX HACK there should be a better way to specify whether 
-					// all faces are in the same file or not
-					Image img;
-	            	// find & load resource data intro stream to allow resource
-					// group changes if required
-					DataStreamPtr dstream = 
-						ResourceGroupManager::getSingleton().openResource(
-							mName, mGroup, true, this);
-
-	                img.load(dstream, ext);
-					// Call internal _loadImages, not loadImage since that's external and 
-					// will determine load status etc again
-					ConstImagePtrList imagePtrs;
-					imagePtrs.push_back(&img);
-					_loadImages( imagePtrs );
-				}
-				else
-				{
-					std::vector<Image> images(6);
-					ConstImagePtrList imagePtrs;
-					static const String suffixes[6] = {"_rt", "_lf", "_up", "_dn", "_fr", "_bk"};
-	
-					for(size_t i = 0; i < 6; i++)
-					{
-						String fullName = baseName + suffixes[i] + "." + ext;
-		            	// find & load resource data intro stream to allow resource
-						// group changes if required
-						DataStreamPtr dstream = 
-							ResourceGroupManager::getSingleton().openResource(
-								fullName, mGroup, true, this);
-	
-						images[i].load(dstream, ext);
-						imagePtrs.push_back(&images[i]);
-					}
-	
-					_loadImages( imagePtrs );
-				}
-            }
-            else
-                OGRE_EXCEPT( Exception::ERR_NOT_IMPLEMENTED, "**** Unknown texture type ****", "GLTexture::load" );
+        // Call internal _loadImages, not loadImage since that's external and 
+        // will determine load status etc again
+        ConstImagePtrList imagePtrs;
+        for (size_t i=0 ; i<loadedImages->size() ; ++i) {
+            imagePtrs.push_back(&(*loadedImages)[i]);
         }
+
+        _loadImages(imagePtrs);
+
     }
-	
+
 	//*************************************************************************
     
     void GLTexture::freeInternalResourcesImpl()
@@ -365,7 +379,7 @@ namespace Ogre {
 			for(size_t mip=0; mip<=getNumMipmaps(); mip++)
 			{
                 GLHardwarePixelBuffer *buf = new GLTextureBuffer(mName, getGLTextureTarget(), mTextureID, face, mip,
-						static_cast<HardwareBuffer::Usage>(mUsage), doSoftware && mip==0);
+						static_cast<HardwareBuffer::Usage>(mUsage), doSoftware && mip==0, mHwGamma, mFSAA);
 				mSurfaceList.push_back(HardwarePixelBufferSharedPtr(buf));
                 
                 /// Check for error
