@@ -37,6 +37,7 @@ Torus Knot Software Ltd.
 #include "OgreWin32GLSupport.h"
 #include "OgreWin32Context.h"
 #include "OgreWindowEventUtilities.h"
+#include "OgreGLPixelFormat.h"
 
 namespace Ogre {
 
@@ -94,9 +95,9 @@ namespace Ogre {
 		HWND parent = 0;
 		String title = name;
 		bool vsync = false;
-		int fsaa = 0;
 		String border;
 		bool outerSize = false;
+		bool hwGamma = false;
 
 		if(miscParams)
 		{
@@ -120,7 +121,10 @@ namespace Ogre {
 				vsync = StringConverter::parseBool(opt->second);
 
 			if ((opt = miscParams->find("FSAA")) != end)
-				fsaa = StringConverter::parseUnsignedInt(opt->second);
+				mFSAA = StringConverter::parseUnsignedInt(opt->second);
+
+			if ((opt = miscParams->find("gamma")) != end)
+				hwGamma = StringConverter::parseBool(opt->second);
 
 			if ((opt = miscParams->find("externalWindowHandle")) != end)
 			{
@@ -155,7 +159,15 @@ namespace Ogre {
 			if ((opt = miscParams->find("displayFrequency")) != end)
 				mDisplayFrequency = StringConverter::parseUnsignedInt(opt->second);
 			if ((opt = miscParams->find("colourDepth")) != end)
+			{
 				mColourDepth = StringConverter::parseUnsignedInt(opt->second);
+				if (!mIsFullScreen)
+				{
+					// make sure we don't exceed desktop colour depth
+					if (mColourDepth > GetDeviceCaps(GetDC(0), BITSPIXEL))
+						mColourDepth = GetDeviceCaps(GetDC(0), BITSPIXEL);
+				}
+			}
 
 			// incompatible with fullscreen
 			if ((opt = miscParams->find("parentWindowHandle")) != end)
@@ -232,11 +244,10 @@ namespace Ogre {
 
 			WindowEventUtilities::_addRenderWindow(this);
 
-			StringUtil::StrStreamType str;
-			str << "Created Win32Window '"
+			LogManager::getSingleton().stream()
+				<< "Created Win32Window '"
 				<< mName << "' : " << mWidth << "x" << mHeight
 				<< ", " << mColourDepth << "bpp";
-			LogManager::getSingleton().logMessage(LML_NORMAL, str.str());
 
 			if (mIsFullScreen)
 			{
@@ -278,21 +289,50 @@ namespace Ogre {
 
 		if (!mIsExternalGLControl)
 		{
-			if (!mGLSupport.selectPixelFormat(mHDC, mColourDepth, fsaa))
+			int testFsaa = mFSAA;
+			bool testHwGamma = hwGamma;
+			bool formatOk = mGLSupport.selectPixelFormat(mHDC, mColourDepth, testFsaa, testHwGamma);
+			if (!formatOk)
 			{
-				if (fsaa == 0)
-					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "selectPixelFormat failed", "Win32Window::create");
+				if (mFSAA > 0)
+				{
+					// try without FSAA
+					testFsaa = 0;
+					formatOk = mGLSupport.selectPixelFormat(mHDC, mColourDepth, testFsaa, testHwGamma);
+				}
+
+				if (!formatOk && hwGamma)
+				{
+					// try without sRGB
+					testHwGamma = false;
+					testFsaa = mFSAA;
+					formatOk = mGLSupport.selectPixelFormat(mHDC, mColourDepth, testFsaa, testHwGamma);
+				}
+
+				if (!formatOk && hwGamma && (mFSAA > 0))
+				{
+					// try without both
+					testHwGamma = false;
+					testFsaa = 0;
+					formatOk = mGLSupport.selectPixelFormat(mHDC, mColourDepth, testFsaa, testHwGamma);
+				}
+
+				if (!formatOk)
+					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
+						"selectPixelFormat failed", "Win32Window::create");
 
-				LogManager::getSingleton().logMessage(LML_NORMAL, "FSAA level not supported, falling back");
-				if (!mGLSupport.selectPixelFormat(mHDC, mColourDepth, 0))
-					OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "selectPixelFormat failed", "Win32Window::create");
 			}
+			// record what gamma option we used in the end
+			// this will control enabling of sRGB state flags when used
+			mHwGamma = testHwGamma;
+			mFSAA = testFsaa;
 		}
 		if (!mIsExternalGLContext)
 		{
 			mGlrc = wglCreateContext(mHDC);
 			if (!mGlrc)
-				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "wglCreateContext", "Win32Window::create");
+				OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, 
+				"wglCreateContext failed: " + translateWGLError(), "Win32Window::create");
 		}
 		if (!wglMakeCurrent(mHDC, mGlrc))
 			OGRE_EXCEPT(Exception::ERR_RENDERINGAPI_ERROR, "wglMakeCurrent", "Win32Window::create");
@@ -504,16 +544,32 @@ namespace Ogre {
 	  }
 	}
 
-	void Win32Window::writeContentsToFile(const String& filename)
+	void Win32Window::copyContentsToMemory(const PixelBox &dst, FrameBuffer buffer)
 	{
-		ImageCodec::ImageData *imgData = new ImageCodec::ImageData();
-		imgData->width = mWidth;
-		imgData->height = mHeight;
-		imgData->depth = 1;
-		imgData->format = PF_BYTE_RGB;
+		if ((dst.left < 0) || (dst.right > mWidth) ||
+			(dst.top < 0) || (dst.bottom > mHeight) ||
+			(dst.front != 0) || (dst.back != 1))
+		{
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+						"Invalid box.",
+						"Win32Window::copyContentsToMemory" );
+		}
+
+		if (buffer == FB_AUTO)
+		{
+			buffer = mIsFullScreen? FB_FRONT : FB_BACK;
+		}
+
+		GLenum format = Ogre::GLPixelUtil::getGLOriginFormat(dst.format);
+		GLenum type = Ogre::GLPixelUtil::getGLOriginDataType(dst.format);
+
+		if ((format == GL_NONE) || (type == 0))
+		{
+			OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS,
+						"Unsupported format.",
+						"Win32Window::copyContentsToMemory" );
+		}
 
-		// Allocate buffer 
-		uchar* pBuffer = new uchar[mWidth * mHeight * 3];
 
 		// Switch context if different from current one
 		RenderSystem* rsys = Root::getSingleton().getRenderSystem();
@@ -522,45 +578,31 @@ namespace Ogre {
 		// Must change the packing to ensure no overruns!
 		glPixelStorei(GL_PACK_ALIGNMENT, 1);
 
-		// Read pixels
-		// I love GL: it does all the locking & colour conversion for us
-		if (mIsFullScreen)
-			glReadBuffer(GL_FRONT);
-		glReadPixels(0,0, mWidth, mHeight, GL_RGB, GL_UNSIGNED_BYTE, pBuffer);
+		glReadBuffer((buffer == FB_FRONT)? GL_FRONT : GL_BACK);
+		glReadPixels((GLint)dst.left, (GLint)dst.top,
+					 (GLsizei)dst.getWidth(), (GLsizei)dst.getHeight(),
+					 format, type, dst.data);
 
 		// restore default alignment
 		glPixelStorei(GL_PACK_ALIGNMENT, 4);
 
-		// Wrap buffer in a memory stream
-		DataStreamPtr stream(new MemoryDataStream(pBuffer, mWidth * mHeight * 3, false));
+		//vertical flip
+		{
+			size_t rowSpan = dst.getWidth() * PixelUtil::getNumElemBytes(dst.format);
+			size_t height = dst.getHeight();
+			uchar *tmpData = new uchar[rowSpan * height];
+			uchar *srcRow = (uchar *)dst.data, *tmpRow = tmpData + (height - 1) * rowSpan;
 
-		// Need to flip the read data over in Y though
-		Image img;
-		img.loadRawData(stream, mWidth, mHeight, imgData->format );
-		img.flipAroundX();
+			while (tmpRow >= tmpData)
+			{
+				memcpy(tmpRow, srcRow, rowSpan);
+				srcRow += rowSpan;
+				tmpRow -= rowSpan;
+			}
+			memcpy(dst.data, tmpData, rowSpan * height);
 
-		MemoryDataStreamPtr streamFlipped(new MemoryDataStream(img.getData(), stream->size(), false));
-
-		// Get codec 
-		size_t pos = filename.find_last_of(".");
-		String extension;
-		if( pos == String::npos )
-			OGRE_EXCEPT(
-			Exception::ERR_INVALIDPARAMS, 
-			"Unable to determine image type for '" + filename + "' - invalid extension.",
-			"Win32Window::writeContentsToFile" );
-
-		while( pos != filename.length() - 1 )
-			extension += filename[++pos];
-
-		// Get the codec
-		Codec * pCodec = Codec::getCodec(extension);
-
-		// Write out
-		Codec::CodecDataPtr ptr(imgData);
-		pCodec->codeToFile(streamFlipped, filename, ptr);
-
-		delete [] pBuffer;
+			delete [] tmpData;
+		}
 	}
 
 	void Win32Window::getCustomAttribute( const String& name, void* pData )
