@@ -36,44 +36,43 @@ import math
 # OGRE_VERTEXCOLOUR_BGRA
 #  workaround for Ogre's vertex colour conversion bug.
 #  Set to 0 for RGBA, 1 for BGRA.
-OGRE_OPENGL_VERTEXCOLOUR = 1
-
-matrixOne = [[1,0,0,0],[0,1,0,0],[0,0,1,0],[0,0,0,1]]
+OGRE_OPENGL_VERTEXCOLOUR = 0
 
 class Vertex:
 	"""
 	"""
 	THRESHOLD = 1e-6
-	def __init__(self, bMesh, bMFace, bIndex, index, parentTransform, armatureExporter=None):
+	def __init__(self, bMesh, bMFace, bIndex, index, fixUpAxis, armatureExporter=None):
 		"""Represents an Ogre vertex.
 		   
 		   @param bIndex Index in the vertex list of the NMFace.
 		   @param index Vertexbuffer position.
-		   @param parentTransform Additional transformation to apply to the vertex.
+		   @param fixUpAxis Additional transformation to apply to the vertex.
 		"""
 		self.bMesh = bMesh
 		# imples position
 		# vertex in basis shape
 		self.bMVert = bMFace.v[bIndex]
-		bKey = self.bMesh.getKey()
+		self.basisPos = self.bMVert.co
+		bKey = self.bMesh.key
 		if (bKey and len(bKey.blocks)):
 			# first shape key is rest position
-			self.bMVert = bKey.blocks[0].data[self.bMVert.index]
+			self.basisPos = bKey.blocks[0].data[self.bMVert.index]
 		## Face properties in Blender
 		self.normal = None
 		self.colourDiffuse = None
-		self.texcoord = None
+		self.texcoords = []
 		## bookkeeping
 		# vertexbuffer position in vertexbuffer
 		self.index = index
-		self.parentTransform = parentTransform
+		self.fixUpAxis = fixUpAxis
 		# implies influences
 		self.armatureExporter = armatureExporter
 		### populated attributes
 		## normal
 		if bMFace.smooth:
 			# key blocks don't have normals
-			self.normal = self._applyParentTransform(bMFace.v[bIndex].no)
+			self.normal = self._applyfixUpAxis(bMFace.v[bIndex].no)
 		else:
 			# create face normal
 			# 1 - 2
@@ -83,14 +82,14 @@ class Vertex:
 			if (bKey and len(bKey.blocks)):
 				# first shape key is rest position
 				blockData = bKey.blocks[0].data
-				v1 = self._applyParentTransform(blockData[bMFace.v[0].index].co)
-				v2 = self._applyParentTransform(blockData[bMFace.v[1].index].co)
-				v3 = self._applyParentTransform(blockData[bMFace.v[2].index].co)
+				v1 = self._applyfixUpAxis(blockData[bMFace.v[0].index])
+				v2 = self._applyfixUpAxis(blockData[bMFace.v[1].index])
+				v3 = self._applyfixUpAxis(blockData[bMFace.v[2].index])
 			else:
 				# self.normal = CrossVecs(bMFace.v[1].co - bMFace.v[0].co, bMFace.v[2].co - bMFace.v[0].co)
-				v1 = self._applyParentTransform(bMFace.v[0].co)
-				v2 = self._applyParentTransform(bMFace.v[1].co)
-				v3 = self._applyParentTransform(bMFace.v[2].co)
+				v1 = self._applyfixUpAxis(bMFace.v[0].co)
+				v2 = self._applyfixUpAxis(bMFace.v[1].co)
+				v3 = self._applyfixUpAxis(bMFace.v[2].co)
 			self.normal = CrossVecs(v2 - v1, v3 - v1)
 		# self.normal.normalize() does not throw ZeroDivisionError exception
 		normalLength = self.normal.length
@@ -100,8 +99,7 @@ class Vertex:
 			Log.getSingleton().logWarning("Error in normalize! Face of mesh \"%s\" too small." % bMesh.name)
 			self.normal = Vector([0,0,0])
 		## colourDiffuse
-		#Mesh#if bMesh.vertexColors:
-		if bMesh.hasVertexColours():
+		if bMesh.vertexColors:
 			bMCol = bMFace.col[bIndex]
 			if OGRE_OPENGL_VERTEXCOLOUR:
 				self.colourDiffuse = (bMCol.b/255.0, bMCol.g/255.0, bMCol.r/255.0, bMCol.a/255.0)
@@ -132,52 +130,63 @@ class Vertex:
 							self.colourDiffuse = (bMCol.r/255.0, bMCol.g/255.0, bMCol.b/255.0, bMCol.a/255.0)
 		## texcoord
 		# origin in OGRE is top-left
-		#Mesh#if bMesh.faceUV:
-		if bMesh.hasFaceUV():
-			self.texcoord = (bMFace.uv[bIndex][0], 1 - bMFace.uv[bIndex][1])
-		#Mesh#elif bMesh.vertexUV:
-		elif bMesh.hasVertexUV():
-			self.texcoord = (self.bMVert.uvco[0], 1 - self.bMVert.uvco[1])
+		activeUVLayer = bMesh.activeUVLayer
+		if activeUVLayer:
+			for uvlayer in bMesh.getUVLayerNames():
+				bMesh.activeUVLayer = uvlayer
+				if bMesh.faceUV:
+					self.texcoords.append((bMFace.uv[bIndex][0], 1 - bMFace.uv[bIndex][1]))
+				elif bMesh.vertexUV:
+					self.texcoords.append((self.bMVert.uvco[0], 1 - self.bMVert.uvco[1]))
+			bMesh.activeUVLayer = activeUVLayer
 		return
 	def __eq__(self, other):
 		"""Tests if this vertex is equal to another vertex in the Ogre sense.
 		
-		   Does no take parentTransform into account!
+		   Does no take fixUpAxis into account!
 		   Also, it does not compare the index.
 		"""
 		isEqual = 0
 		# compare index, normal, colourDiffuse and texcoord
-		if (self.bMVert.index == other.bMVert.index):
+		if (self.bMVert.index != other.bMVert.index):
+			# different Blender vertex
 			pass
 		elif ((self.normal - other.normal).length > Vertex.THRESHOLD):
+			# normals don't match
 			pass
-		elif ((self.texcoord and not(other.texcoord)) or 
-			(not(self.texcoord) and other.texcoord)):
-			pass
-		elif ((math.fabs(self.texcoord[0] - other.texcoord[0]) > Vertex.THRESHOLD)
-			or (math.fabs(self.texcoord[1] - other.texcoord[1]) > Vertex.THRESHOLD)):
+		elif (not(self.matchTexCoords(other))):
+			# texture coordinates do not match
 			pass
 		elif ((self.colourDiffuse and not(other.colourDiffuse)) or 
 			(not(self.colourDiffuse) and other.colourDiffuse)):
+			# mixed existence of vertex colours
 			pass
-		elif ((math.fabs(self.colourDiffuse[0] - other.colourDiffuse[0]) > Vertex.THRESHOLD)
+		elif (self.colourDiffuse and
+			((math.fabs(self.colourDiffuse[0] - other.colourDiffuse[0]) > Vertex.THRESHOLD)
 			or (math.fabs(self.colourDiffuse[1] - other.colourDiffuse[1]) > Vertex.THRESHOLD)
 			or (math.fabs(self.colourDiffuse[2] - other.colourDiffuse[2]) > Vertex.THRESHOLD)
-			or (math.fabs(self.colourDiffuse[3] - other.colourDiffuse[3]) > Vertex.THRESHOLD)):
+			or (math.fabs(self.colourDiffuse[3] - other.colourDiffuse[3]) > Vertex.THRESHOLD))):
+			# vertex colours exist but do not match
 			pass
 		else:
 			isEqual = 1
 		return isEqual
+	def matchTexCoords(self, other):
+		if (len(self.texcoords) != len(other.texcoords)):
+			return False
+		else:
+			for id in range(len(self.texcoords)):
+				if ((math.fabs(self.texcoords[id][0] - other.texcoords[id][0]) > Vertex.THRESHOLD)
+				or (math.fabs(self.texcoords[id][1] - other.texcoords[id][1]) > Vertex.THRESHOLD)):
+					return False
+		return True
 	def hasDiffuseColours(self):
 		available = False
 		if self.colourDiffuse is not None:
 			available = True
 		return available
 	def nTextureCoords(self):
-		nCoords = 0
-		if self.texcoord is not None:
-			nCoords += 1
-		return nCoords
+		return len(self.texcoords)
 	def writePosition(self, fileObject, indentation=0):
 		fileObject.write(indent(indentation) + "<position x=\"%.6f\" y=\"%.6f\" z=\"%.6f\"/>\n" \
 			% tuple(self.getPosition()))
@@ -192,9 +201,9 @@ class Vertex:
 				% self.colourDiffuse)
 		return
 	def writeTexcoord(self, fileObject, indentation=0):
-		if self.texcoord:
+		for id in range(len(self.texcoords)):
 			fileObject.write(indent(indentation) + "<texcoord u=\"%.6f\" v=\"%.6f\"/>\n"\
-				% self.texcoord)
+				% self.texcoords[id])
 		return
 	def writeVertex(self, fileObject, indentation=0):
 		fileObject.write(indent(indentation) + "<vertex>\n")
@@ -228,12 +237,13 @@ class Vertex:
 			Log.getSingleton().logWarning("Vertex without bone assignment!")
 		elif (nAssignments  > 4):
 			Log.getSingleton().logWarning("Vertex with more than 4 bone assignments!")
-		# weightSum > 1.0 seems to be often the case.
-		# TODO: check whether OGRE requires that it is <= 1.0
-		# TODO: check whether Blender takes this as a scale factor, i.e., 
-		#   influence is sum_i bone_i*weight_i, or if weights are normalized to sum_i weight_i == 1
-		#if (weightSum > 1.0):
-		#	Log.getSingleton().logWarning("Vertex with sum of bone assignment weights > 1!")
+		# TODO: weightSum normalization
+		# Ogre::Mesh::_rationaliseBoneAssignments always normalises the sum of
+		# weights per vertex to be 1.0. However, in Blender weightSum > 1.0 and
+		# weightSum < 1.0 seems to be often the case.
+		#
+		#if (abs(weightSum - 1.0) > THRESHOLD):
+		#	Log.getSingleton().logWarning("Vertex with non-convex bone assignment weights!")
 		return
 	def getIndex(self):
 		return self.index
@@ -242,30 +252,30 @@ class Vertex:
 	def getPosition(self):
 		"""Returns position vector of the rest position.
 		"""
-		return self._applyParentTransform(self.bMVert.co)
+		return self._applyfixUpAxis(self.basisPos)
 	def getCurrentFramePosition(self, bDeformedNMesh):
 		"""Returns position of this vertex in the current frame of the possibly deformed mesh.
 		"""
-		return self._applyParentTransform(bDeformedNMesh.verts[self.bMVert.index].co)
+		return self._applyfixUpAxis(bDeformedNMesh.verts[self.bMVert.index].co)
 	def getCurrentFrameRelativePosition(self, bDeformedNMesh):
 		"""Returns relative position of this vertex in the current frame of the possibly deformed mesh.
 		"""
 		return (self.getCurrentFramePosition(bDeformedNMesh) - self.getPosition())
-	def _applyParentTransform(self, vector):
+	def _applyfixUpAxis(self, vector):
 		"""Applies transformation to threedimensional vector.
 		"""
-		vec = Vector(vector)
-		vec.resize4D()
-		vec = vec * self.parentTransform
-		vec.resize3D()
+		if (self.fixUpAxis):
+			vec = Vector(vector.x, vector.z, -vector.y)
+		else:
+			return vector
 		return vec
 
 class VertexManager:
 	"""
 	"""
-	def __init__(self, bMesh, parentTransform, armatureExporter=None):
+	def __init__(self, bMesh, fixUpAxis, armatureExporter=None):
 		self.bMesh = bMesh
-		self.parentTransform = parentTransform
+		self.fixUpAxis = fixUpAxis
 		# needed for boneassignments
 		self.armatureExporter = armatureExporter
 		# key: index, value: list of vertices with same MVert
@@ -287,7 +297,7 @@ class VertexManager:
 		   @param bIndex Index in the vertex list of the MFace.
 		   @return Corresponding vertex.
 		"""
-		vertex = Vertex(self.bMesh, bMFace, bIndex, len(self.vertexList), self.parentTransform, self.armatureExporter)
+		vertex = Vertex(self.bMesh, bMFace, bIndex, len(self.vertexList), self.fixUpAxis, self.armatureExporter)
 		if self.vertexDict.has_key(bMFace.v[bIndex].index):
 			# check Ogre vertices for that Blender vertex
 			vertexList = self.vertexDict[bMFace.v[bIndex].index]
@@ -317,8 +327,10 @@ class VertexManager:
 			firstVertex = self.vertexList[0]
 			if firstVertex.hasDiffuseColours():
 				fileObject.write(" colours_diffuse=\"true\"")
-			if (firstVertex.nTextureCoords() > 0):
-				fileObject.write(" texture_coords=\"1\" texture_coord_dimensions_0=\"2\"")
+			# set texture coordinate count.
+			coords = firstVertex.nTextureCoords()
+			if (coords > 0):
+				fileObject.write(" texture_coords=\"%d\"" % coords)
 		fileObject.write(">\n")
 		for vertex in self.vertexList:
 			vertex.writeVertex(fileObject, indentation + 2)
@@ -348,7 +360,7 @@ class VertexManager:
 class Submesh:
 	"""Ogre submesh.
 	"""
-	def __init__(self, bMesh, material, index, parentTransform, armatureExporter = None):
+	def __init__(self, bMesh, material, index, fixUpAxis, armatureExporter = None):
 		"""Constructor.
 		
 		   @param index Index of submesh in submeshes list.
@@ -356,9 +368,9 @@ class Submesh:
 		self.bMesh = bMesh
 		self.materialName = material.getName()
 		self.index = index
-		self.parentTransform = parentTransform
+		self.fixUpAxis = fixUpAxis
 		self.armatureExporter = armatureExporter
-		self.vertexManager = VertexManager(self.bMesh, self.parentTransform, self.armatureExporter)
+		self.vertexManager = VertexManager(self.bMesh, self.fixUpAxis, self.armatureExporter)
 		# list of (tuple of vertice indices)
 		self.faces =[]
 		return
@@ -426,9 +438,9 @@ class Submesh:
 class SubmeshManager:
 	"""
 	"""
-	def __init__(self, bMesh, parentTransform, armatureExporter=None):
+	def __init__(self, bMesh, fixUpAxis, armatureExporter=None):
 		self.bMesh = bMesh
-		self.parentTransform = parentTransform
+		self.fixUpAxis = fixUpAxis
 		self.armatureExporter = armatureExporter
 		# key: material name, value: Submesh
 		self.submeshDict = {}
@@ -446,7 +458,7 @@ class SubmeshManager:
 		else:
 			# return new Submesh
 			index = len(self.submeshList)
-			submesh = Submesh(self.bMesh, material, index, self.parentTransform, self.armatureExporter)
+			submesh = Submesh(self.bMesh, material, index, self.fixUpAxis, self.armatureExporter)
 			self.submeshDict[material.getName()] = submesh
 			self.submeshList.append(submesh)
 		return submesh
@@ -474,7 +486,7 @@ class Pose:
 	"""
 	"""
 	THRESHOLD = 1e-7
-	def __init__(self, bKeyBlock, submesh, index, parentTransform):
+	def __init__(self, bKeyBlock, submesh, index, fixUpAxis):
 		"""Constructor.
 		
 		   @param index Index of pose in poses list.
@@ -482,13 +494,13 @@ class Pose:
 		self.bKeyBlock = bKeyBlock
 		self.submesh = submesh
 		self.index = index
-		self.parentTransform = parentTransform
+		self.fixUpAxis = fixUpAxis
 		# list of pose offset tuples (vertexIndex, deltaX, deltaY, deltaZ)
 		self.poseoffsetList	= []
 		# calculate poseoffsets
 		poseVertexList = self.bKeyBlock.data
 		for vertex in self.submesh.getVertexManager():
-			offset = self._applyParentTransform(poseVertexList[vertex.getMVert().index]) \
+			offset = self._applyfixUpAxis(poseVertexList[vertex.getMVert().index]) \
 				 - vertex.getPosition()
 			if (offset.length > Pose.THRESHOLD):
 				self.poseoffsetList.append((vertex.getIndex(), offset.x, offset.y, offset.z))
@@ -515,33 +527,32 @@ class Pose:
 					% poseoffset)
 			fileObject.write(indent(indentation) + "</pose>\n")		
 		return
-	def _applyParentTransform(self, vector):
+	def _applyfixUpAxis(self, vector):
 		"""Applies transformation to threedimensional vector.
 		"""
-		vec = Vector(vector)
-		vec.resize4D()
-		vec = vec * self.parentTransform
-		vec.resize3D()
+		if (self.fixUpAxis):
+			vec = Vector(vector.x, vector.z, -vector.y)
+		else:
+			return vector
 		return vec
 	
 class PoseManager:
 	"""
 	"""
-	def __init__(self, bMesh, submeshManager, parentTransform):
+	def __init__(self, bMesh, submeshManager, fixUpAxis):
 		self.bMesh = bMesh
 		self.submeshManager = submeshManager
-		self.parentTransform = parentTransform
 		# key: submesh, value: poseList
 		self.poseListDict = {}
 		self.poseList = []
 		# create poses
 		# each keyblock creates a pose for every submesh
-		bKey = self.bMesh.getKey()
+		bKey = self.bMesh.key
 		if bKey:
 			for bKeyBlock in bKey.blocks:
 				for submesh in self.submeshManager:
 					index = len(self.poseList)
-					pose = Pose(bKeyBlock, submesh, index, self.parentTransform)
+					pose = Pose(bKeyBlock, submesh, index, fixUpAxis)
 					if (pose.nPoseoffsets() > 0):
 						# add nonempty pose to list and dict
 						self.poseList.append(pose)
@@ -712,7 +723,8 @@ class MorphAnimation(VertexAnimation):
 		timeList.sort()
 		for time in timeList:
 			Blender.Set('curframe', frameNumberDict[time])
-			bDeformedNMesh = Blender.NMesh.GetRawFromObject(bObject.getName())
+			#~ bDeformedNMesh = Blender.NMesh.GetRawFromObject(bObject.getName())
+			bDeformedNMesh = bObject.getData(mesh=True)
 			for track in self.trackList:
 				track.addKeyframe(bDeformedNMesh, time)
 		return
@@ -766,9 +778,9 @@ class VertexAnimationExporter:
 		return
 	def hasAnimation(self):
 		return (len(self.morphAnimationList) or len(self.poseAnimationList))
-	def export(self, parentTransform):
+	def export(self, fixUpAxis):
 		# generate poses
-		self.poseManager = PoseManager(self.meshExporter.getObject().getData(), self.meshExporter.getSubmeshManager(), parentTransform)
+		self.poseManager = PoseManager(self.meshExporter.getObject().getData(mesh=True), self.meshExporter.getSubmeshManager(), fixUpAxis)
 		if self.hasAnimation():
 			# sample animations
 			animationNameList = []
@@ -830,7 +842,7 @@ class MeshExporter:
 	   Exports mesh, armature and animations to Ogre XML resp. script files. Materials are
 	   exported to a MaterialManager.
 	"""
-	def __init__(self, bObject):
+	def __init__(self, bObject, skeletonUseMeshName):
 		"""
 		"""
 		# mesh
@@ -842,26 +854,32 @@ class MeshExporter:
 		self.armatureExporter = None
 		parent = GetArmatureObject(self.bObject)
 		if (parent is not None):
-			self.armatureExporter = ArmatureExporter(self.bObject, parent)
+			self.armatureExporter = ArmatureExporter(self.bObject, parent, skeletonUseMeshName)
 		# populated on export
 		self.submeshManager = None
 		return
-	def export(self, dir, materialManager=MaterialManager(), parentTransform=Matrix(*matrixOne), colouredAmbient=False, gameEngineMaterials=False, convertXML=False):
+	def export(self, dir, materialManager=MaterialManager(), fixUpAxis=True, exportMesh=True, colouredAmbient=False, convertXML=False):
 		# leave editmode
 		editmode = Blender.Window.EditMode()
 		if editmode:
 			Blender.Window.EditMode(0)
-		Log.getSingleton().logInfo("Exporting mesh \"%s\"" % self.getName())
-		## export possible armature
-		if self.armatureExporter:
-			self.armatureExporter.export(dir, parentTransform, convertXML)
+
+		if exportMesh:
+			Log.getSingleton().logInfo("Exporting mesh \"%s\"" % self.getName())
+			## export possible armature
+			if self.armatureExporter:
+				self.armatureExporter.export(dir, fixUpAxis, convertXML)
+
 		## export meshdata
-		self._generateSubmeshes(parentTransform, materialManager, colouredAmbient, gameEngineMaterials)
-		## export vertex animations
-		self.vertexAnimationExporter.export(parentTransform)
-		## write files
-		self._write(dir, convertXML)
-		## cleanup
+		self._generateSubmeshes(fixUpAxis, materialManager, colouredAmbient, exportMesh)
+
+		if exportMesh:
+			## export vertex animations
+			self.vertexAnimationExporter.export(fixUpAxis)
+			## write files
+			self._write(dir, convertXML)
+			## cleanup
+
 		self.submeshManager = None
 		# reenter editmode
 		if editmode:
@@ -877,15 +895,15 @@ class MeshExporter:
 		return self.armatureExporter
 	def getSubmeshManager(self):
 		return self.submeshManager
-	def _generateSubmeshes(self, parentTransform, materialManager, colouredAmbient, gameEngineMaterials):
+	def _generateSubmeshes(self, fixUpAxis, materialManager, colouredAmbient, exportMesh):
 		"""Generates submeshes of the mesh.
 		"""
 		#NMesh# Blender.Mesh.Mesh does not provide access to mesh shape keys, use Blender.NMesh.NMesh
-		bMesh = self.bObject.getData()
-		self.submeshManager = SubmeshManager(bMesh, parentTransform, self.armatureExporter)
+		bMesh = self.bObject.getData(mesh=True)
+		self.submeshManager = SubmeshManager(bMesh, fixUpAxis, self.armatureExporter)
 		for bMFace in bMesh.faces:
-			faceMaterial = materialManager.getMaterial(bMesh, bMFace, colouredAmbient, gameEngineMaterials)
-			if faceMaterial:
+			faceMaterial = materialManager.getMaterial(bMesh, bMFace, colouredAmbient)
+			if faceMaterial and exportMesh:
 				# append face to submesh
 				self.submeshManager.getSubmesh(faceMaterial).addFace(bMFace)
 		return
