@@ -19,6 +19,11 @@
 
 namespace OgreMayaExporter
 {
+	OgreExporter::~OgreExporter()
+	{
+		exit();
+	}
+	// Restore the scene to a state previous to the export, clean up memory and exit
 	void OgreExporter::exit()
 	{
 		// Restore active selection list
@@ -26,17 +31,22 @@ namespace OgreMayaExporter
 		// Restore current time
 		MAnimControl::setCurrentTime(m_curTime);
 		// Free memory
-		if (m_pMesh)
-			delete m_pMesh;
-		if (m_pMaterialSet)
-			delete m_pMaterialSet;
+		delete m_pMesh;
+		m_pMesh = 0;
+		delete m_pMaterialSet;
+		m_pMaterialSet = 0;
 		// Close output files
 		m_params.closeFiles();
 		std::cout.flush();
 	}
 
+	// Execute the command
 	MStatus OgreExporter::doIt(const MArgList& args)
 	{
+		// clean up
+		delete m_pMesh;
+		delete m_pMaterialSet;
+
 		// Parse the arguments.
 		m_params.parseArgs(args);
 		// Create output files
@@ -225,8 +235,8 @@ namespace OgreMayaExporter
 		for (unsigned int i=0; i<dagPath.childCount(); i++)
 		{
 			MObject child = dagPath.child(i);
-			MDagPath childPath;
-			stat = MDagPath::getAPathTo(child,childPath);
+			 MDagPath childPath = dagPath;
+			stat = childPath.push(child);
 			if (MS::kSuccess != stat)
 			{
 				std::cout << "Error retrieving path to child " << i << " of: " << dagPath.fullPathName().asChar();
@@ -295,6 +305,7 @@ namespace OgreMayaExporter
 		// write translation data
 		m_params.outCameras << "\ttranslation\n";
 		m_params.outCameras << "\t{\n";
+		//translateX
 		m_params.outCameras << "\t\tx ";
 		plug = cameraTransform->findPlug("translateX");
 		if (plug.isConnected() && m_params.exportCamerasAnim)
@@ -322,6 +333,7 @@ namespace OgreMayaExporter
 			plug.getValue(dist);
 			m_params.outCameras << "= " << dist << "\n";
 		}
+		//translateY
 		m_params.outCameras << "\t\ty ";
 		plug = cameraTransform->findPlug("translateY");
 		if (plug.isConnected() && m_params.exportCamerasAnim)
@@ -349,6 +361,7 @@ namespace OgreMayaExporter
 			plug.getValue(dist);
 			m_params.outCameras << "= " << dist << "\n";
 		}
+		//translateZ
 		m_params.outCameras << "\t\tz ";
 		plug = cameraTransform->findPlug("translateZ");
 		if (plug.isConnected() && m_params.exportCamerasAnim)
@@ -382,6 +395,7 @@ namespace OgreMayaExporter
 		m_params.outCameras << "\trotation\n";
 		m_params.outCameras << "\t{\n";
 		m_params.outCameras << "\t\tx ";
+		//rotateX
 		plug = cameraTransform->findPlug("rotateX");
 		if (plug.isConnected() && m_params.exportCamerasAnim)
 		{
@@ -408,6 +422,7 @@ namespace OgreMayaExporter
 			plug.getValue(angle);
 			m_params.outCameras << "= " << angle.asDegrees() << "\n";
 		}
+		//rotateY
 		m_params.outCameras << "\t\ty ";
 		plug = cameraTransform->findPlug("rotateY");
 		if (plug.isConnected() && m_params.exportCamerasAnim)
@@ -435,6 +450,7 @@ namespace OgreMayaExporter
 			plug.getValue(angle);
 			m_params.outCameras << "= " << angle.asDegrees() << "\n";
 		}
+		//rotateZ
 		m_params.outCameras << "\t\tz ";
 		plug = cameraTransform->findPlug("rotateZ");
 		if (plug.isConnected() && m_params.exportCamerasAnim)
